#################################################################################
##
##   R package rgarch by Alexios Ghalanos Copyright (C) 2008, 2009, 2010, 2011
##   This file is part of the R package rgarch.
##
##   The R package rgarch is free software: you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   The R package rgarch is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
#################################################################################

# Package Highest Level Virtual Class
setClass("rGARCH","VIRTUAL")

#----------------------------------------------------------------------------------
# univariate spec class
#----------------------------------------------------------------------------------
setClass("GARCHspec", contains = c("rGARCH", "VIRTUAL"))
setClass("uGARCHspec", contains = c("GARCHspec", "VIRTUAL"))
#-----------------------------------------------------

# Family GARCH Models (Hentschel)
setClass("fGARCHspec",
		representation(mean.model = "vector",
				variance.model = "vector",
				distribution.model = "vector",
				optimization.model = "vector"),
		contains = "uGARCHspec")

# standard GARCH
setClass("sGARCHspec",
		representation(mean.model = "vector",
				variance.model = "vector",
				distribution.model = "vector",
				optimization.model = "vector"),
		contains = "uGARCHspec")


# exponential GARCH
setClass("eGARCHspec",
		representation(mean.model = "vector",
				variance.model = "vector",
				distribution.model = "vector",
				optimization.model = "vector"),
		contains = "uGARCHspec")

# GJR GARCH
setClass("gjrGARCHspec",
		representation(mean.model = "vector",
				variance.model = "vector",
				distribution.model = "vector",
				optimization.model = "vector"),
		contains = "uGARCHspec")
		
# apARCH
setClass("apARCHspec",
		representation(mean.model = "vector",
				variance.model = "vector",
				distribution.model = "vector",
				optimization.model = "vector"),
		contains = "uGARCHspec")

# Integrated GARCH
setClass("iGARCHspec",
		representation(mean.model = "vector",
				variance.model = "vector",
				distribution.model = "vector",
				optimization.model = "vector"),
		contains = "uGARCHspec")

#----------------------------------------------------------------------------------
# univariate filter class
#----------------------------------------------------------------------------------
setClass("GARCHfilter", contains = c("rGARCH", "VIRTUAL"))
setClass("uGARCHfilter", contains = c("GARCHfilter", "VIRTUAL"))
#setClass("mGARCHfilter", contains = c("GARCHfilter", "VIRTUAL"))
#-----------------------------------------------------


# Family GARCH Models (Hentschel)
setClass("fGARCHfilter",
		representation(filter = "vector",
				model = "vector"),
		contains = "uGARCHfilter")

# Standard GARCH
setClass("sGARCHfilter",
		representation(filter = "vector",
				model = "vector"),
		contains = "uGARCHfilter")

# exponential GARCH
setClass("eGARCHfilter",
		representation(filter = "vector",
				model = "vector"),
		contains = "uGARCHfilter")

# exponential GARCH
setClass("gjrGARCHfilter",
		representation(filter = "vector",
				model = "vector"),
		contains = "uGARCHfilter")

# apARCH
setClass("apARCHfilter",
		representation(filter = "vector",
				model = "vector"),
		contains = "uGARCHfilter")

# Integrated GARCH
setClass("iGARCHfilter",
		representation(filter = "vector",
				model = "vector"),
		contains = "uGARCHfilter")

#----------------------------------------------------------------------------------
# univariate fit class
#----------------------------------------------------------------------------------
setClass("GARCHfit", contains = c("rGARCH", "VIRTUAL"))
setClass("uGARCHfit", contains = c("GARCHfit", "VIRTUAL"))
#setClass("mGARCHfit", contains = c("GARCHfit", "VIRTUAL"))
#-----------------------------------------------------


# Family GARCH Models (Hentschel)
setClass("fGARCHfit",
		representation(fit = "vector",
				model = "vector"),
		contains = "uGARCHfit")

# Standard GARCH
setClass("sGARCHfit",
		representation(fit = "vector",
				model = "vector"),
		contains = "uGARCHfit")

# exponential GARCH
setClass("eGARCHfit",
		representation(fit = "vector",
				model = "vector"),
		contains = "uGARCHfit")

# GJR GARCH
setClass("gjrGARCHfit",
		representation(fit = "vector",
				model = "vector"),
		contains = "uGARCHfit")

# apARCH
setClass("apARCHfit",
		representation(fit = "vector",
				model = "vector"),
		contains = "uGARCHfit")

# Integrated GARCH
setClass("iGARCHfit",
		representation(fit = "vector",
				model = "vector"),
		contains = "uGARCHfit")
#----------------------------------------------------------------------------------
# univariate simulation class
#----------------------------------------------------------------------------------
setClass("GARCHsim", contains = c("rGARCH", "VIRTUAL"))
setClass("uGARCHsim", contains = c("GARCHsim", "VIRTUAL"))
#setClass("mGARCHsim", contains=c("GARCHsim", "VIRTUAL"))
#-----------------------------------------------------

# Family GARCH Models (Hentschel)
setClass("fGARCHsim",
		representation(simulation = "vector",
				model = "vector",
				seed = "integer"),
		contains = "uGARCHsim")

# Standard GARCH
setClass("sGARCHsim",
		representation(simulation = "vector",
				model = "vector",
				seed = "integer"),
		contains = "uGARCHsim")

# exponential GARCH
setClass("eGARCHsim",
		representation(simulation = "vector",
				model = "vector",
				seed = "integer"),
		contains = "uGARCHsim")

# exponential GARCH
setClass("gjrGARCHsim",
		representation(simulation = "vector",
				model = "vector",
				seed = "integer"),
		contains = "uGARCHsim")

# apARCH
setClass("apARCHsim",
		representation(simulation = "vector",
				model = "vector",
				seed = "integer"),
		contains = "uGARCHsim")

# Integrated GARCH
setClass("iGARCHsim",
		representation(simulation = "vector",
				model = "vector",
				seed = "integer"),
		contains = "uGARCHsim")
		
#----------------------------------------------------------------------------------
# univariate forecast class (extends filter class)
#----------------------------------------------------------------------------------
setClass("GARCHforecast", contains = c("rGARCH", "VIRTUAL"))
setClass("uGARCHforecast", contains = c("GARCHforecast", "VIRTUAL"))
#setClass("mGARCHforecast", contains = c("GARCHforecast", "VIRTUAL"))
#-----------------------------------------------------

# Family GARCH Models (Hentschel)
setClass("fGARCHforecast",
		representation(forecast = "vector",
				model = "vector"),
		contains = c( "fGARCHfilter", "uGARCHforecast"))

# Standard GARCH
setClass("sGARCHforecast",
		representation(forecast = "vector",
				model = "vector"),
		contains = c( "sGARCHfilter", "uGARCHforecast"))

# exponential GARCH
setClass("eGARCHforecast",
		representation(forecast = "vector",
				model = "vector"),
		contains = c( "eGARCHfilter", "uGARCHforecast"))

# GJR GARCH
setClass("gjrGARCHforecast",
		representation(forecast = "vector",
				model = "vector"),
		contains = c( "gjrGARCHfilter", "uGARCHforecast"))
		
# apARCH
setClass("apARCHforecast",
		representation(forecast = "vector",
				model = "vector"),
		contains = c( "apARCHfilter", "uGARCHforecast"))

# Integrated GARCH
setClass("iGARCHforecast",
		representation(forecast = "vector",
				model = "vector"),
		contains = c( "iGARCHfilter", "uGARCHforecast"))

#----------------------------------------------------------------------------------
# univariate path simulation class
#----------------------------------------------------------------------------------
setClass("GARCHpath", contains = c("rGARCH", "VIRTUAL"))
setClass("uGARCHpath", contains = c("GARCHpath", "VIRTUAL"))
#setClass("mGARCHpath", contains = c("GARCHpath", "VIRTUAL"))
#-----------------------------------------------------

# Family GARCH Models (Hentschel)
setClass("fGARCHpath",
		representation(path = "vector",
				seed = "integer"),
		contains = "uGARCHpath")

# Standard GARCH
setClass("sGARCHpath",
		representation(path = "vector",
				seed = "integer"),
		contains = "uGARCHpath")

# exponential GARCH
setClass("eGARCHpath",
		representation(path = "vector",
				seed = "integer"),
		contains = "uGARCHpath")

# exponential GARCH
setClass("gjrGARCHpath",
		representation(path = "vector",
				seed = "integer"),
		contains = "uGARCHpath")

# apARCH
setClass("apARCHpath",
		representation(path = "vector",
				seed = "integer"),
		contains = "uGARCHpath")

# Integrated GARCH
setClass("iGARCHpath",
		representation(path = "vector",
				seed = "integer"),
		contains = "uGARCHpath")

#----------------------------------------------------------------------------------
# univariate garch roll class
#----------------------------------------------------------------------------------
setClass("GARCHroll", contains = c("rGARCH", "VIRTUAL"))
setClass("uGARCHroll", contains = c("GARCHroll", "VIRTUAL"))

#setClass("mGARCHroll", contains = c("GARCHroll", "VIRTUAL"))
#-----------------------------------------------------


setClass("sGARCHroll",
		representation(roll = "vector",
				forecast = "vector"),
		contains = c("sGARCHspec","uGARCHroll"))

setClass("fGARCHroll",
		representation(roll = "vector",
				forecast = "vector"),
		contains = c("fGARCHspec","uGARCHroll"))

setClass("eGARCHroll",
		representation(roll = "vector",
				forecast = "vector"),
		contains = c("eGARCHspec","uGARCHroll"))

setClass("gjrGARCHroll",
		representation(roll = "vector",
				forecast = "vector"),
		contains = c("gjrGARCHspec","uGARCHroll"))

setClass("iGARCHroll",
		representation(roll = "vector",
				forecast = "vector"),
		contains = c("iGARCHspec","uGARCHroll"))

setClass("apARCHroll",
		representation(roll = "vector",
				forecast = "vector"),
		contains = c("apARCHspec","uGARCHroll"))

setClass("sGARCHroll",
		representation(roll = "vector",
				forecast = "vector"),
		contains = c("sGARCHspec","uGARCHroll"))

#----------------------------------------------------------------------------------
# univariate garch parameter distribution (by simulation)
#----------------------------------------------------------------------------------
setClass("GARCHdistribution", contains = c("rGARCH", "VIRTUAL"))


setClass("uGARCHdistribution",
		representation(dist = "vector",
				truecoef = "matrix",
				spec = "uGARCHspec"),
		contains =  "GARCHdistribution")

#----------------------------------------------------------------------------------
# univariate garch bootstrap forecast distribution
#----------------------------------------------------------------------------------
setClass("GARCHboot", contains = c("rGARCH", "VIRTUAL"))

setClass("uGARCHboot",
		representation(
				fseries = "matrix",
				fsigma = "matrix",
				bcoef = "data.frame",
				model = "vector",
				spec = "uGARCHspec",
				forc = "uGARCHforecast"),
		contains =  "GARCHboot")
#-----------------------------------------------------

#----------------------------------------------------------------------------------
# univariate garch forecast performance measures
#----------------------------------------------------------------------------------
setClass("GARCHtests", contains = c("rGARCH", "VIRTUAL"))

setClass("uGARCHfpm",
		representation(
				seriesfpm = "vector",
				sigmafpm = "vector",
				forecasts = "vector",
				realized = "vector",
				details = "vector"),
		contains =  "GARCHtests")
#-----------------------------------------------------

#----------------------------------------------------------------------------------
# multiple spec/fit/filter/forecast garch methods (used in 2-stage extension multivariate models)
#----------------------------------------------------------------------------------

# Multiple Spec List Class
setClass("uGARCHmultispec", 
		representation(spec = "vector",
				type = "character"),
		contains = "uGARCHspec")

.validspeclist = function(object){
	all(unlist(lapply(object@spec, FUN = function(x) is(x, "uGARCHspec"))))
}

setValidity("uGARCHmultispec", .validspeclist)

# Multiple Fit ACD List Class
setClass("uGARCHmultifit", 
		representation(fit = "vector",
				model = "vector"),
		contains = "uGARCHfit")

.validfitlist = function(object){
	all(unlist(lapply(object@fit, FUN = function(x) is(x, "uGARCHfit"))))
}

setValidity("uGARCHmultifit", .validfitlist)

# Multiple Fit ACD List Class
setClass("uGARCHmultifilter", 
		representation(filter = "vector",
				model = "vector"),
		contains = "uGARCHfilter")

.validfilterlist = function(object){
	all(unlist(lapply(object@filter, FUN = function(x) is(x, "uGARCHfilter"))))
}

setValidity("uGARCHmultifilter", .validfilterlist)




# Multiple Forecast ACD List Class
setClass("uGARCHmultiforecast", 
		representation(forecast = "vector",
				model = "vector"),
		contains = "uGARCHforecast")

.validforecastlist = function(object){
	all(unlist(lapply(object@forecast, FUN = function(x) is(x, "uGARCHforecast"))))
}

setValidity("uGARCHmultiforecast", .validforecastlist)

###################################################################################
#----------------------------------------------------------------------------------
# Multivariate Class Models
#----------------------------------------------------------------------------------
setClass("mGARCHspec", contains = c("GARCHspec", "VIRTUAL"))
setClass("mGARCHfit", contains = c("GARCHfit", "VIRTUAL"))
setClass("mGARCHfilter", contains = c("GARCHfilter", "VIRTUAL"))
setClass("mGARCHforecast", contains = c("GARCHforecast", "VIRTUAL"))
setClass("mGARCHsim", contains = c("GARCHsim", "VIRTUAL"))
setClass("mGARCHroll", contains = c("GARCHroll", "VIRTUAL"))
#----------------------------------------------------------------------------------
