#################################################################################
##
##   R package rgarch by Alexios Ghalanos Copyright (C) 2008, 2009, 2010, 2011
##   This file is part of the R package rgarch.
##
##   The R package rgarch is free software: you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   The R package rgarch is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
#################################################################################

# The starting parameters for the mean equation should be estimated first
# The specification will depend on which aspects of the mean equation are
# chosen i.e. arfima/inmean/arma/
# For the time being, we ignore fixed parameters in the initialization process
# and treat all as non-fixed (although arima function provides for a fixed pars
# option, I am not sure what to do with the lm (I think the 'offset' options does 
# something...)

# Care should be taken with using arima0 and arima with regards to
# 1: parameter names (arima returns xreg while arima0 returns mexdata),
# i.e. one returns the dataframe name and the other the colname.
# 2: using fixed parameters and the subsequent warnings (not used right now)

.arfimastart = function(data, garchenv)
{
	mmodel = get("mmodel", garchenv)
	omodel = get("omodel", garchenv)
	dmodel = get("dmodel", garchenv)
	armap = mmodel$armaOrder[1]
	armaq = mmodel$armaOrder[2]
	arfima = mmodel$arfima
	include.mex = mmodel$include.mex
	mexdata = mmodel$mexdata
	include.mean = mmodel$include.mean
	mxreg = numeric()
	
	start.pars = omodel$start.pars
	opt.pars = omodel$opt.pars
	fixed.pars = omodel$fixed.pars
	
	dscale = get("dscale", garchenv)
	opt.names = names(opt.pars)
	fixed.names = names(fixed.pars)
	LB = vector(mode = "numeric", length = length(opt.names))
	names(LB) = opt.names
	UB = vector(mode = "numeric", length = length(opt.names))
	names(UB) = opt.names
	pos = omodel$pos.matrix
	start.names = names(start.pars)
	dbounds = .DistributionBounds(distribution = dmodel$distribution)
	
	
	if(include.mex){
		mexdata = mexdata
		mxn = dim(as.matrix(mexdata))[2]
	} else{
		mexdata = NULL
		mxn = 0
	}
	mu = 0
	ar = rep(0, armap)
	ma = rep(0, armaq)
	inmean = 0
	
	if( armap!=0 | armaq!=0 ){
		ttemp = arima(data, order = c(armap, 0, armaq), include.mean = include.mean, xreg = mexdata, method = "CSS")
		fit.mean = ttemp$coef
		#res=ttemp$residuals
		if(include.mean){
			mu = fit.mean["intercept"]
		}
		if(armap!=0) ar = fit.mean[c(paste("ar",1:armap,sep=""))]
		if(armaq!=0) ma = fit.mean[c(paste("ma",1:armaq,sep=""))]
		if(include.mex){
			i = which(substr(names(fit.mean), 1, 7) == "mexdata")
			mxreg = fit.mean[i]
		}
	} else{
		y = data
		if( include.mex ){
			mxn = dim(as.matrix(mexdata))[2]
			fit.mean = lm(y~mexdata)
			i = which(substr(names(fit.mean$coef), 1, 7) == "mexdata")
			mu = fit.mean$coef["(Intercept)"]
			mxreg = fit.mean$coef[i]
			#res=as.numeric(fit.mean$residuals)
		} else{
			mu = 0
			res = data
		}
	}
	
	if( pos[1,3] ){
		LB["mu"] = -100*abs(mean(data))
		UB["mu"] =  100*abs(mean(data))
		if( is.null(start.pars$mu) ) opt.pars["mu"] = mu else opt.pars["mu"] = start.pars$mu[1]/dscale
		if( any(substr(fixed.names, 1, 2) == "mu") ){
			opt.pars["mu"] = as.numeric(fixed.pars$mu)
			LB["mu"] = fixed.pars$mu
			UB["mu"] = fixed.pars$mu
		}
	}
	
	# ar
	if( pos[2,3] ){
		arnames = paste("ar", 1:armap, sep = "")
		LB[arnames] = -1+TinY
		UB[arnames] =  1-TinY
		opt.pars[arnames] = as.numeric(ar)
		if( any(substr(start.names, 1, 2) == "ar") ){
			j = which(substr(start.names, 1, 2) == "ar")
			armatch = charmatch(start.names[j], arnames)
			opt.pars[arnames[armatch]] = as.numeric(start.pars[j])
		}
		if( any(substr(fixed.names, 1, 2) == "ar") ){
			j = which(substr(fixed.names, 1, 2) == "ar")
			armatch = charmatch(fixed.names[j], arnames)
			opt.pars[arnames[armatch]] = as.numeric(fixed.pars[j])
			LB[arnames[armatch]] = as.numeric(fixed.pars[j])
			UB[arnames[armatch]] = as.numeric(fixed.pars[j])
		}
	}
	# ma
	if( pos[3,3] ){
		manames = paste("ma", 1:armaq, sep = "")
		LB[manames] = -1+TinY
		UB[manames] =  1-TinY
		opt.pars[manames] = as.numeric(ma)
		if( any(substr(start.names, 1, 2) == "ma") ){
			j = which(substr(start.names, 1, 2) == "ma")
			mamatch = charmatch(start.names[j], manames)
			opt.pars[manames[mamatch]] = as.numeric(start.pars[j])
		}
		if( any(substr(fixed.names, 1, 2) == "ma") ){
			j = which(substr(fixed.names, 1, 2) == "ma")
			mamatch = charmatch(fixed.names[j], manames)
			opt.pars[manames[mamatch]] = as.numeric(fixed.pars[j])
			LB[manames[mamatch]] = as.numeric(fixed.pars[j])
			UB[manames[mamatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	# arfima
	if( pos[4,3] ){
		LB["darfima"] =-1
		UB["darfima"] = 1
		if(is.null(start.pars$darfima)) opt.pars["darfima"] = .rsfit(data) - 0.5 else opt.pars["darfima"] = start.pars$darfima[1]
		if( any(substr(fixed.names, 1, 7) == "darfima") ){
			opt.pars["darfima"] = as.numeric(fixed.pars$darfima)
			LB["darfima"] = fixed.pars$darfima
			UB["darfima"] = fixed.pars$darfima
		}
	}
	
	# exogenous regressors
	if( pos[5,3] ){
		mxnames = paste("mxreg", 1:mxn, sep = "")
		opt.pars[mxnames] = as.numeric(mxreg)
		LB[mxnames] = as.numeric(abs(opt.pars[paste("mxreg", 1:mxn, sep = "")])) * -100
		UB[mxnames] = as.numeric(abs(opt.pars[paste("mxreg", 1:mxn, sep = "")])) *  100
		if( any(substr(start.names, 1, 5) == "mxreg") ){
			j = which(substr(start.names, 1, 5) == "mxreg")
			mxmatch = charmatch(start.names[j], mxnames)
			opt.pars[mxnames[mxmatch]] = as.numeric(start.pars[j])
		}
		if( any(substr(fixed.names, 1, 5) == "mxreg") ){
			j = which(substr(fixed.names, 1, 5) == "mxreg")
			mxmatch = charmatch(fixed.names[j], mxnames)
			opt.pars[mxnames[mxmatch]] = as.numeric(fixed.pars[j])
			LB[mxnames[mxmatch]] = as.numeric(fixed.pars[j])
			UB[mxnames[mxmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	if( pos[6,3] ){
		.sd = sd(data)
		LB["sigma"] = .eps
		UB["sigma"] = 100 * .sd
		if( is.null(start.pars$sigma) ) opt.pars["sigma"] = .sd else opt.pars["sigma"] = start.pars$sigma[1]
		if( any(substr(fixed.names, 1, 5) == "sigma") ){
			opt.pars["sigma"] = as.numeric(fixed.pars$sigma)
			LB["sigma"] = fixed.pars$sigma
			UB["sigma"] = fixed.pars$sigma
		}
	}
	
	if( pos[7,3] ){
		LB["dlambda"] = dbounds$dlambda.LB
		UB["dlambda"] = dbounds$dlambda.UB
		if( is.null(start.pars$dlambda) ) opt.pars["dlambda"] = dbounds$dlambda else opt.pars["dlambda"] = start.pars$dlambda[1]
		if( any(substr(fixed.names, 1, 7 ) == "dlambda")){
			opt.pars["dlambda"] = as.numeric(fixed.pars$dlambda)
			LB["dlambda"] = fixed.pars$dlambda
			UB["dlambda"] = fixed.pars$dlambda
		}
	}
	if( pos[8,3] ){
		LB["skew"] = dbounds$skew.LB
		UB["skew"] = dbounds$skew.UB
		if( is.null(start.pars$skew) ) opt.pars["skew"] = dbounds$skew else opt.pars["skew"] = start.pars$skew[1]
		if( any(substr(fixed.names, 1, 4) == "skew") ){
			opt.pars["skew"] = as.numeric(fixed.pars$skew)
			LB["skew"] = fixed.pars$skew
			UB["skew"] = fixed.pars$skew
		}
	}
	
	# shape
	if( pos[9,3] ){
		if( is.null(start.pars$shape) ) opt.pars["shape"] = dbounds$shape else opt.pars["shape"] = start.pars$shape[1]
		LB["shape"] = dbounds$shape.LB
		UB["shape"] = dbounds$shape.UB
		if( any(substr(fixed.names, 1, 5) == "shape") ){
			opt.pars["shape"] = as.numeric(fixed.pars$shape)
			LB["shape"] = fixed.pars$shape
			UB["shape"] = fixed.pars$shape
		}
	}
	
	modelLB = LB
	modelUB = UB
	
	return( list( pars = opt.pars, modelLB = modelLB, modelUB = modelUB ) )
}


.meqstartpars = function(data, model, submodel, garchenv)
{
	# Case 1 garchInMean yes:
	mmodel = get("mmodel", garchenv)
	vmodel = get("vmodel", garchenv)
	omodel = get("omodel", garchenv)
	dmodel = get("dmodel", garchenv)
	armap = mmodel$armaOrder[1]
	armaq = mmodel$armaOrder[2]
	im = mmodel$inMeanType
	# im = 1 (sigma), im = 2 (variance)
	pow = vmodel$pow
	# we modify to work with sigma
	# whereas in c-code it works with var
	garchInMean = mmodel$garchInMean
	arfima = mmodel$arfima
	include.mex = mmodel$include.mex
	mexdata = mmodel$mexdata
	include.mean = mmodel$include.mean
	mxreg = numeric()
	
	if(include.mex){
		mexdata = mexdata
		mxn = dim(as.matrix(mexdata))[2]
		# easier to NULL the names of the data rather than search them by name later
		# (search now is 'mexdata'..1:mxn)
		colnames(mexdata) = NULL
	} else{
		mexdata = NULL
	}
	mu = 0
	ar = rep(0, armap)
	ma = rep(0, armaq)
	inmean = 0
	h = 0
	
	# get the sigma vector should it be needed (garchInMean)
	if(garchInMean){
		ctrl = list(eval.max = 1000, trace = 0, iter.max = 1000)
		tempspec = ugarchspec(variance.model = list(model = model, garchOrder = vmodel$garchOrder, 
						submodel = submodel), mean.model = list(armaOrder = c(armap,armaq),
						include.mean = include.mean, garchInMean = FALSE), distribution.model = "norm")
		# we try first with stationarity conditions enforced
		tempfit = ugarchfit(data = data, spec = tempspec, solver="solnp", solver.control = ctrl, fit.control = list(stationarity = 1))
		if(tempfit@fit$convergence!=0){
			tempfit = ugarchfit(data, tempspec, solver = "solnp", solver.control = ctrl, fit.control = list(stationarity = 0))
			if(tempfit@fit$convergence!=0) stop("\nugarchfit-->error: could not find appropriate starting values for recursion\n")
		}
		h = tempfit@fit$sigma
	}
	# arima without garchInMean
	if(!(garchInMean) && (armap!=0 | armaq!=0)){
		ttemp = arima(data, order = c(armap, 0, armaq), include.mean = include.mean, xreg = mexdata, method = "CSS")
		fit.mean = ttemp$coef
		#res=ttemp$residuals
		if(include.mean){
			mu = fit.mean["intercept"]
		}
		if(armap!=0) ar = fit.mean[c(paste("ar",1:armap,sep=""))]
		if(armaq!=0) ma = fit.mean[c(paste("ma",1:armaq,sep=""))]
		if(include.mex){
			i = which(substr(names(fit.mean), 1, 7) == "mexdata")
			mxreg = fit.mean[i]
		}
	}
	
	# arima with garchInMean	
	if((garchInMean) && (armap+armaq)>0){
			mexdata = cbind(mexdata, h^im)
			colnames(mexdata) = paste("xreg", 1:dim(mexdata)[2],sep="")
			mxn = dim(mexdata)[2]
			ttemp = arima0(data, order = c(armap, 0, armaq), include.mean = include.mean, xreg = mexdata)
			fit.mean = ttemp$coef
			if(include.mean){
				mu = fit.mean["intercept"]
			}
			if(armap!=0) ar = fit.mean[c(paste("ar",1:armap,sep=""))]
			if(armaq!=0) ma = fit.mean[c(paste("ma",1:armaq,sep=""))]
			i = which(substr(names(fit.mean), 1, 4) == "xreg")
			z = length(i) # at a minimum it is 2 ex+inmean
			if(include.mex){
				mxreg = fit.mean[i[1:(z-1)]]
			}
			inmean = fit.mean[i[z]]
	} 
	
	# lm for garchInMean without arma
	if((garchInMean) && (armap == 0 && armaq == 0)){
			mexdata = cbind(mexdata, h^im)
			mxn = dim(mexdata)[2]
			y = data
			if(include.mean){
				fit.mean = lm(y~mexdata)
				mu = fit.mean$coef["(Intercept)"]
				i = which(substr(names(fit.mean$coef), 1, 7) == "mexdata")
				z = length(i) # at a minimum it is 2 ex+inmean
				if(include.mex){
					mxreg = fit.mean$coef[i[1:(z-1)]]
				}
				inmean = fit.mean$coef[i[z]]
				#res=as.numeric(fit.mean$residuals)
			} else{
				fit.mean = lm(y~mxreg3-1)
				if(include.mex) mxreg = fit.mean$coef[c(paste("mexdata",1:(mxn-1),sep=""))]			
				inmean = fit.mean$coef[c(paste("mexdata",mxn,sep=""))]
				#res=as.numeric(fit.mean$residuals)
			}
		} 
	
	if(!(garchInMean) && armap == 0 && armaq == 0){
		y = data
		if(include.mex){
			mxn = dim(as.matrix(mexdata))[2]
			fit.mean = lm(y~mexdata)
			i = which(substr(names(fit.mean$coef), 1, 7) == "mexdata")
			mu = fit.mean$coef["(Intercept)"]
			mxreg = fit.mean$coef[i]
			#res=as.numeric(fit.mean$residuals)
		} else{
			mu = 0
			#res=(data-mu)
		}
	}
	sol = list(mu = mu, ar = ar, ma = ma, inmean = inmean, mxreg = mxreg, h = h)
	return(sol)
}

# common to all specifications is the mean equation:
.meqstart = function(data, meqspars, mmodel, opt.pars, start.pars, fixed.pars, garchenv)
{
	dscale = get("dscale", garchenv)
	opt.names = names(opt.pars)
	fixed.names = names(fixed.pars)
	LB = vector(mode = "numeric", length = length(opt.names))
	names(LB) = opt.names
	UB = vector(mode = "numeric", length = length(opt.names))
	names(UB) = opt.names
	armap = get("mmodel", garchenv)$armaOrder[1]
	armaq = get("mmodel", garchenv)$armaOrder[2]
	mxn = get("mmodel", garchenv)$mxn
	pos = get("omodel", garchenv)$pos.matrix
	start.names = names(start.pars)
	# this is where we fix the bounds for the fixed parameters
	# fill and then the fixed.names to overwrite it...also for the
	# bounds
	
	if(pos[1,3]){
		LB["mu"] = -100*abs(mean(data))
		UB["mu"] =  100*abs(mean(data))
		if(is.null(start.pars$mu)) opt.pars["mu"] = meqspars$mu else opt.pars["mu"] = start.pars$mu[1]/dscale
		if(any(substr(fixed.names, 1, 2)=="mu")){
			opt.pars["mu"]=as.numeric(fixed.pars$mu)
			LB["mu"] = fixed.pars$mu
			UB["mu"] = fixed.pars$mu
		}
	}
	
	# ar
	if(pos[2,3]){
		arnames=paste("ar",1:armap,sep="")
		LB[arnames] = -1+TinY
		UB[arnames] =  1-TinY
	 	opt.pars[arnames] = as.numeric(meqspars$ar)
		if(any(substr(start.names, 1, 2)=="ar")){
			j = which(substr(start.names, 1, 2)=="ar")
			armatch = charmatch(start.names[j],arnames)
			opt.pars[arnames[armatch]]=as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 2)=="ar")){
			j = which(substr(fixed.names, 1, 2)=="ar")
			armatch = charmatch(fixed.names[j],arnames)
			opt.pars[arnames[armatch]]=as.numeric(fixed.pars[j])
			LB[arnames[armatch]]=as.numeric(fixed.pars[j])
			UB[arnames[armatch]]=as.numeric(fixed.pars[j])
		}
	}
	# ma
	if(pos[3,3]){
		manames = paste("ma",1:armaq,sep="")
		LB[manames] = -1+TinY
		UB[manames] =  1-TinY
		opt.pars[manames] = as.numeric(meqspars$ma)
		if(any(substr(start.names, 1, 2)=="ma")){
			j = which(substr(start.names, 1, 2)=="ma")
			mamatch = charmatch(start.names[j],manames)
			opt.pars[manames[mamatch]]=as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 2)=="ma")){
			j = which(substr(fixed.names, 1, 2)=="ma")
			mamatch = charmatch(fixed.names[j],manames)
			opt.pars[manames[mamatch]]=as.numeric(fixed.pars[j])
			LB[manames[mamatch]]=as.numeric(fixed.pars[j])
			UB[manames[mamatch]]=as.numeric(fixed.pars[j])
		}
	}
	
	# garch in mean
	if(pos[4,3]){
		LB["inmean"] = -100
		UB["inmean"] = 100
		if(is.null(start.pars$inmean)) opt.pars["inmean"] = meqspars$inmean else opt.pars["inmean"] = start.pars$inmean[1]
		if(any(substr(fixed.names, 1, 6)=="inmean")){
			opt.pars["inmean"]=as.numeric(fixed.pars$inmean)
			LB["inmean"]=fixed.pars$inmean
			UB["inmean"]=fixed.pars$inmean
		}
	}
	
	# arfima
	if(pos[5,3]){
		LB["darfima"] =-1
		UB["darfima"] = 1
		if(is.null(start.pars$darfima)) opt.pars["darfima"] = .rsfit(data)-0.5 else opt.pars["darfima"] = start.pars$darfima[1]
		if(any(substr(fixed.names, 1, 7)=="darfima")){
			opt.pars["darfima"]=as.numeric(fixed.pars$darfima)
			LB["darfima"]=fixed.pars$darfima
			UB["darfima"]=fixed.pars$darfima
		}
	}
	
	# exogenous regressors
	if(pos[6,3]){
		mxnames = paste("mxreg",1:mxn,sep="")
		opt.pars[mxnames] = as.numeric(meqspars$mxreg)
		LB[mxnames] = as.numeric(abs(opt.pars[paste("mxreg",1:mxn,sep="")]))*-100
		UB[mxnames] = as.numeric(abs(opt.pars[paste("mxreg",1:mxn,sep="")]))*100
		if(any(substr(start.names, 1, 5)=="mxreg")){
			j = which(substr(start.names, 1, 5)=="mxreg")
			mxmatch = charmatch(start.names[j],mxnames)
			opt.pars[mxnames[mxmatch]]=as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5)=="mxreg")){
			j = which(substr(fixed.names, 1, 5)=="mxreg")
			mxmatch = charmatch(fixed.names[j],mxnames)
			opt.pars[mxnames[mxmatch]]=as.numeric(fixed.pars[j])
			LB[mxnames[mxmatch]]=as.numeric(fixed.pars[j])
			UB[mxnames[mxmatch]]=as.numeric(fixed.pars[j])
		}
	}		
	return( list( opt.pars = opt.pars, LB = LB, UB = UB ) )
}

# starting parameters s.t. specification
.garchstart = function(data, model, submodel, garchenv)
{
	# first we call the mean equation start parameter estimation
	# and assign it to the running environment
	meqspars = .meqstartpars(data = data, model = model, submodel = submodel, garchenv)
	assign("meqspars", meqspars, envir = garchenv)
	
	# We call the appropriate model since different models require different setups
	ans = switch(model,
			sGARCH = .sgarchstart(data, model, submodel, garchenv),
			fGARCH = .fgarchstart(data, model, submodel, garchenv),
			gjrGARCH = .gjrgarchstart(data, model, submodel, garchenv),
			apARCH = .aparchstart(data, model, submodel, garchenv),
			eGARCH = .egarchstart(data, model, submodel, garchenv),
			iGARCH = .igarchstart(data, model, submodel, garchenv))
			#anstGARCH = .anstgarchstart(data, model, submodel, garchenv))
	return(ans)
}

# sGARCH model start parameters
.sgarchstart = function(data, model, submodel, garchenv)
{
	# extract required sets
	dscale = get("dscale", garchenv)
	mmodel = get("mmodel", garchenv)
	vmodel = get("vmodel",garchenv)
	dmodel = get("dmodel", garchenv)
	omodel = get("omodel", garchenv)
	start.pars = omodel$start.pars
	opt.pars = omodel$opt.pars
	fixed.pars = omodel$fixed.pars
	pos = omodel$pos.matrix
	meqspars = get("meqspars", garchenv)
	mxn = mmodel$mxn
	vexdata = vmodel$vexdata
	opt.names = names(opt.pars)
	fixed.names = names(fixed.pars)
	start.names = names(start.pars)
	garchp = vmodel$garchOrder[1]
	garchq = vmodel$garchOrder[2]
	vxn = vmodel$vxn
	# get the common to all mean equation parameter start vector
	# and bounds
	meqx = .meqstart(data, meqspars, mmodel, opt.pars, start.pars, fixed.pars, garchenv)
	pars = meqx$opt.pars
	LB = meqx$LB
	UB = meqx$UB
	
	# get the sgarch parameter start vector and bounds
	if(pos[7,3]){
		LB["omega"] = var(data)/100000
		UB["omega"] = var(data)*100000
		if(is.null(start.pars$omega)) pars["omega"] = (var(data, na.rm = TRUE))/1000 else 
			pars["omega"] = start.pars$omega[1]/dscale
		if(any(substr(fixed.names, 1, 5) == "omega")){
			pars["omega"] = as.numeric(fixed.pars$omega)
			LB["omega"] = fixed.pars$omega
			UB["omega"] = fixed.pars$omega
		}
	
	}
	# garchp (alpha)
	# - potentially we could have a problem of the user enters a garch model with
	# p|q>1 and enters a high fixed parameter for either making violating the alpha+beta<1 rule
	# and disrupting the solver...

	if(pos[8,3]){
		gpnames = paste("alpha",1:garchp,sep="")
		LB[gpnames] = 0
		UB[gpnames] = 1-TinY
		pars[gpnames] = rep(0.05/garchp,garchp)
		if(any(substr(start.names, 1, 5) == "alpha")){
			j = which(substr(start.names, 1, 5) == "alpha")
			gpmatch = charmatch(start.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "alpha")){
			j = which(substr(fixed.names, 1, 5) == "alpha")
			gpmatch = charmatch(fixed.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			LB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			UB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	# garchq (beta)
	if(pos[9,3]){
		gqnames = paste("beta",1:garchq,sep="")
		LB[gqnames] = 0
		UB[gqnames] = 1-TinY
		pars[gqnames] = rep(0.9/garchq,garchq)
		if(any(substr(start.names, 1, 4) == "beta")){
			j = which(substr(start.names, 1, 4) == "beta")
			gqmatch = charmatch(start.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 4) == "beta")){
			j = which(substr(fixed.names, 1, 4) == "beta")
			gqmatch = charmatch(fixed.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			LB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			UB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	# external regressors
	if(pos[10,3]){
		vxnames = paste("vxreg",1:vxn,sep="")
		LB[vxnames] = 0
		UB[vxnames] = 100
		pars[vxnames] = rep(TinY, vxn)
		if(any(substr(start.names, 1, 5) == "vxreg")){
			j = which(substr(start.names, 1, 5) == "vxreg")
			vxmatch = charmatch(start.names[j],vxnames)
			pars[vxnames[vxmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "vxreg")){
			j = which(substr(fixed.names, 1, 5) == "vxreg")
			vxmatch = charmatch(fixed.names[j],vxnames)
			pars[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
			LB[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
			UB[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
		}
	}
	dbounds = .DistributionBounds(distribution=dmodel$distribution)
	# skew

	if(pos[11,3]){
		LB["dlambda"] = dbounds$dlambda.LB
		UB["dlambda"] = dbounds$dlambda.UB
		if(is.null(start.pars$dlambda)) pars["dlambda"] = dbounds$dlambda else pars["dlambda"] = start.pars$dlambda[1]
		if(any(substr(fixed.names, 1, 7) == "dlambda")){
			pars["dlambda"] = as.numeric(fixed.pars$dlambda)
			LB["dlambda"] = fixed.pars$dlambda
			UB["dlambda"] = fixed.pars$dlambda
		}
	}
	if(pos[12,3]){
		LB["skew"] = dbounds$skew.LB
		UB["skew"] = dbounds$skew.UB
		if(is.null(start.pars$skew)) pars["skew"] = dbounds$skew else pars["skew"] = start.pars$skew[1]
		if(any(substr(fixed.names, 1, 4) == "skew")){
			pars["skew"] = as.numeric(fixed.pars$skew)
			LB["skew"] = fixed.pars$skew
			UB["skew"] = fixed.pars$skew
		}
	}

	# shape
	if(pos[13,3]){
		if(is.null(start.pars$shape)) pars["shape"] = dbounds$shape else pars["shape"] = start.pars$shape[1]
		LB["shape"] = dbounds$shape.LB
		UB["shape"] = dbounds$shape.UB
		if(any(substr(fixed.names, 1, 5) == "shape")){
			pars["shape"] = as.numeric(fixed.pars$shape)
			LB["shape"] = fixed.pars$shape
			UB["shape"] = fixed.pars$shape
		}
	}
	modelLB = LB
	modelUB = UB
	return(list(pars = pars, modelLB = modelLB, modelUB = modelUB))
}

# Hentchel Family Model Initialization:
# model specification
# modified Hentschel model to include indicator power for switching
.fgarchModel = function(model)
{
	ans = switch(model,
			GARCH    = list(parameters = 
							list(lambda = 2, delta = 2, gamma1 = 0, gamma2 = 0, fk = 0),
							indicator = c(0, 0, 0, 0, 0), garchtype = 1),
			TGARCH   = list(parameters = 
							list(lambda = 1, delta = 1, gamma1 = 0.05, gamma2 = 0, fk = 0),
							indicator = c(0, 0, 1, 0, 0), garchtype = 2),
			AVGARCH  = list(parameters = 
							list(lambda = 1, delta = 1, gamma1 = 0.02, gamma2 = 0.05, fk = 0), 
							indicator = c(0, 0, 1, 1, 0), garchtype = 3),
			NGARCH   = list(parameters = 
							list(lambda = 2, delta = 0, gamma1 = 0, gamma2 = 0, fk = 1), 
							indicator = c(1, 0, 0, 0, 0), garchtype = 4),
			NAGARCH  = list(parameters = 
							list(lambda = 2, delta = 2, gamma1 = 0, gamma2 = 0.05, fk = 0), 
							indicator = c(0, 0, 0, 1, 0), garchtype = 5),
			APARCH   = list(parameters = 
							list(lambda = 1, delta = 0, gamma1 = 0.05, gamma2 = 0, fk = 1), 
							indicator = c(1, 0, 1, 0, 0), garchtype = 6),
			ALLGARCH = list(parameters = 
							list(lambda = 2, delta = 0, gamma1 = 0.05, gamma2 = 0.05, fk = 1), 
							indicator = c(1, 0, 1, 1, 0), garchtype = 7),
			GJRGARCH = list(parameters = 
							list(lambda = 2, delta = 2, gamma1 = 0.05, gamma2 = 0, fk = 0), 
							indicator = c(0, 0, 1, 0, 0), garchtype = 8),
			EGARCH   = list(parameters = 
							list(lambda = eps, delta = 1, gamma1 = 0.05, gamma2 = 0, fk = 0), 
							indicator = c(0, 0, 1, 0, 0), garchtype = 0))
	return(ans)
}

.fmodelBounds = function(model)
{
	# lambda, delta, gamma1, gamma2, fk
	ans = switch(model,
			GARCH    = list( LB = c(    2, 2,       0,   0,  0 ), UB = c(   2, 2,      0,  0, 0 ) ),
			TGARCH   = list( LB = c(    1, 1, -1+TinY,   0,  0 ), UB = c(   1, 1, 1-TinY,  0, 0 ) ),
			AVGARCH  = list( LB = c(    1, 1, -1+TinY, -10,  0 ), UB = c(   1, 1, 1-TinY, 10, 0 ) ),
			NGARCH   = list( LB = c( 0.01, 0,       0,   0,  1 ), UB = c(   4, 0,      0,  0, 1 ) ),
			NAGARCH  = list( LB = c(    2, 2,       0, -10,  0 ), UB = c(   2, 2,      0, 10, 0 ) ),
			APARCH   = list( LB = c( 0.01, 0, -1+TinY,   0,  1 ), UB = c(   4, 0, 1-TinY,  0, 1 ) ),
			ALLGARCH = list( LB = c( 0.01, 0, -1+TinY, -10,  1 ), UB = c(   4, 0, 1-TinY, 10, 1 ) ),
			GJRGARCH = list( LB = c(    2, 2, -1+TinY,   0,  0 ), UB = c(   2, 2, 1-TinY,  0, 0 ) ),
			EGARCH   = list( LB = c( 0.01, 1,      -1,   0,  0 ), UB = c( eps, 1,      1,  0, 0 ) ))
	return(ans)
}

# fGARCH model start parameters
.fgarchstart = function(data, model, submodel, garchenv)
{
	# extract required sets
	dscale = get("dscale", garchenv)
	mmodel = get("mmodel", garchenv)
	vmodel = get("vmodel", garchenv)
	dmodel = get("dmodel", garchenv)
	omodel = get("omodel", garchenv)
	start.pars = omodel$start.pars
	opt.pars = omodel$opt.pars
	fixed.pars = omodel$fixed.pars
	pos = omodel$pos.matrix
	meqspars = get("meqspars", garchenv)
	mxn = mmodel$mxn
	vexdata = vmodel$vexdata
	opt.names = names(opt.pars)
	fixed.names = names(fixed.pars)
	start.names = names(start.pars)
	garchp = vmodel$garchOrder[1]
	garchq = vmodel$garchOrder[2]
	vxn = vmodel$vxn
	fbounds = vmodel$fbounds
	fpars = vmodel$fpars
	# get the common to all mean equation parameter start vector
	# and bounds
	meqx = .meqstart(data, meqspars, mmodel, opt.pars, start.pars, fixed.pars, garchenv)
	pars = meqx$opt.pars
	LB = meqx$LB
	UB = meqx$UB
	
	if(pos[7,3]){
		LB["omega"] = eps
		UB["omega"] = var(data)*100000
		if(is.null(start.pars$omega)) pars["omega"] = (var(data, na.rm = TRUE))/1000 else 
			pars["omega"] = start.pars$omega[1]/dscale
		if(any(substr(fixed.names, 1, 5) == "omega")){
			pars["omega"] = as.numeric(fixed.pars$omega)
			LB["omega"] = fixed.pars$omega
			UB["omega"] = fixed.pars$omega
		}
		
	}
	
	# garchp (alpha)
	if(pos[8,3]){
		gpnames = paste("alpha",1:garchp,sep="")
		LB[gpnames] = eps
		UB[gpnames] = 1-TinY
		pars[gpnames] = rep(0.05/garchp, garchp)
		if(any(substr(start.names, 1, 5) == "alpha")){
			j = which(substr(start.names, 1, 5) == "alpha")
			gpmatch = charmatch(start.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "alpha")){
			j = which(substr(fixed.names, 1, 5) == "alpha")
			gpmatch = charmatch(fixed.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			LB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			UB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	if(pos[8,3] && pos[9,3]){
		gpnames = paste("gamma1", 1:garchp, sep="")
		LB[gpnames] = fbounds$LB[3]
		UB[gpnames] = fbounds$UB[3]
		pars[gpnames] = fpars$gamma1
		if(any(substr(start.names, 1, 6) == "gamma1")){
			j = which(substr(start.names, 1, 6) == "gamma1")
			gpmatch = charmatch(start.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 6) == "gamma1")){
			j = which(substr(fixed.names, 1, 6) == "gamma1")
			gpmatch = charmatch(fixed.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			LB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			UB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	if(pos[8,3] && pos[10,3]){
		gpnames = paste("gamma2",1:garchp,sep="")
		LB[gpnames] = fbounds$LB[4]
		UB[gpnames] = fbounds$UB[4]
		pars[gpnames] = fpars$gamma2
		if(any(substr(start.names, 1, 6) == "gamma2")){
			j = which(substr(start.names, 1, 6) == "gamma2")
			gpmatch = charmatch(start.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 6) == "gamma2")){
			j = which(substr(fixed.names, 1, 6) == "gamma2")
			gpmatch = charmatch(fixed.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			LB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			UB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	# garchq (beta)
	if(pos[11,3]){
		gqnames = paste("beta", 1:garchq,sep="")
		LB[gqnames] = eps
		UB[gqnames] = 1-TinY
		pars[gqnames] = rep(0.9/garchq,garchq)
		if(any(substr(start.names, 1, 4) == "beta")){
			j = which(substr(start.names, 1, 4) == "beta")
			gqmatch = charmatch(start.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 4) == "beta")){
			j = which(substr(fixed.names, 1, 4) == "beta")
			gqmatch = charmatch(fixed.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			LB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			UB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	if(pos[12,3]){
		LB["lambda"] = fbounds$LB[1]
		UB["lambda"] = fbounds$UB[1]
		if(is.null(start.pars$lambda)) pars["lambda"] = fpars$lambda else pars["lambda"] = start.pars$lambda[1]
		if(any(substr(fixed.names, 1, 6) == "lambda")){
			pars["lambda"] = as.numeric(fixed.pars$lambda)
			LB["lambda"] = fixed.pars$lambda
			UB["lambda"] = fixed.pars$lambda
		}
	}
	
	if(pos[8,3] && pos[13,3]){
		LB["delta"] = fbounds$LB[2]
		UB["delta"] = fbounds$UB[2]
		if(is.null(start.pars$delta)) pars["delta"] = fpars$delta else pars["delta"] = start.pars$delta[1]
		if(any(substr(fixed.names, 1, 5) == "delta")){
			pars["delta"] = as.numeric(fixed.pars$delta)
			LB["delta"] = fixed.pars$delta
			UB["delta"] = fixed.pars$delta
		}
	}
	
	# external regressors
	if(pos[14,3]){
		vxnames = paste("vxreg",1:vxn,sep="")
		LB[vxnames] = 0
		UB[vxnames] = 100
		pars[vxnames] = rep(TinY, vxn)
		if(any(substr(start.names, 1, 5) == "vxreg")){
			j = which(substr(start.names, 1, 5) == "vxreg")
			vxmatch = charmatch(start.names[j],vxnames)
			pars[vxnames[vxmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "vxreg")){
			j = which(substr(fixed.names, 1, 5) == "vxreg")
			vxmatch = charmatch(fixed.names[j],vxnames)
			pars[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
			LB[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
			UB[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	if(pos[15,3]){
		LB["dlambda"] = dbounds$dlambda.LB
		UB["dlambda"] = dbounds$dlambda.UB
		if(is.null(start.pars$dlambda)) pars["dlambda"] = dbounds$dlambda else pars["dlambda"] = start.pars$dlambda[1]
		if(any(substr(fixed.names, 1, 7) == "dlambda")){
			pars["dlambda"] = as.numeric(fixed.pars$dlambda)
			LB["dlambda"] = fixed.pars$dlambda
			UB["dlambda"] = fixed.pars$dlambda
		}
	}
	
	# get the parameter start vector and bounds for the distribution
	dbounds = .DistributionBounds(distribution=dmodel$distribution)
	# skew
	if(pos[16,3]){
		LB["skew"] = dbounds$skew.LB
		UB["skew"] = dbounds$skew.UB
		if(is.null(start.pars$skew)) pars["skew"] = dbounds$skew else pars["skew"] = start.pars$skew[1]
		if(any(substr(fixed.names, 1, 4) == "skew")){
			pars["skew"] = as.numeric(fixed.pars$skew)
			LB["skew"] = fixed.pars$skew
			UB["skew"] = fixed.pars$skew
		}
	}
	# shape
	if(pos[17,3]){
		if(is.null(start.pars$shape)) pars["shape"] = dbounds$shape else pars["shape"] = start.pars$shape[1]
		LB["shape"] = dbounds$shape.LB
		UB["shape"] = dbounds$shape.UB
		if(any(substr(fixed.names, 1, 5) == "shape")){
			pars["shape"] = as.numeric(fixed.pars$shape)
			LB["shape"] = fixed.pars$shape
			UB["shape"] = fixed.pars$shape
		}
	}
	modelLB = LB
	modelUB = UB
	return(list(pars = pars, modelLB = modelLB, modelUB = modelUB))
}



.egarchstart = function(data, model, submodel, garchenv)
{
	# extract required sets
	dscale = get("dscale", garchenv)
	mmodel = get("mmodel", garchenv)
	vmodel = get("vmodel", garchenv)
	dmodel = get("dmodel", garchenv)
	omodel = get("omodel", garchenv)
	start.pars = omodel$start.pars
	opt.pars = omodel$opt.pars
	fixed.pars = omodel$fixed.pars
	pos = omodel$pos.matrix
	meqspars = get("meqspars", garchenv)
	mxn = mmodel$mxn
	vexdata = vmodel$vexdata
	opt.names = names(opt.pars)
	fixed.names = names(fixed.pars)
	start.names = names(start.pars)
	garchp = vmodel$garchOrder[1]
	garchq = vmodel$garchOrder[2]
	vxn = vmodel$vxn
	# get the common to all mean equation parameter start vector
	# and bounds
	meqx = .meqstart(data, meqspars, mmodel, opt.pars, start.pars, fixed.pars, garchenv)
	pars = meqx$opt.pars
	LB = meqx$LB
	UB = meqx$UB
	
	if(pos[7,3]){
		LB["omega"]=-1000
		UB["omega"]= 1000
		if(is.null(start.pars$omega)) pars["omega"] = (var(data, na.rm = TRUE))/1000 else 
			pars["omega"] = start.pars$omega[1]/dscale
		if(any(substr(fixed.names, 1, 5) == "omega")){
			pars["omega"] = as.numeric(fixed.pars$omega)
			LB["omega"] = fixed.pars$omega
			UB["omega"] = fixed.pars$omega
		}
		
	}
	
	# garchp (alpha)
	if(pos[8,3]){
		gpnames = paste("alpha",1:garchp,sep="")
		LB[gpnames] = -10
		UB[gpnames] = 10
		pars[gpnames] = rep(0.4, garchp)
		if(any(substr(start.names, 1, 5) == "alpha")){
			j = which(substr(start.names, 1, 5) == "alpha")
			gpmatch = charmatch(start.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "alpha")){
			j = which(substr(fixed.names, 1, 5) == "alpha")
			gpmatch = charmatch(fixed.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			LB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			UB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	# gamma
	if(pos[9,3]){
		gpnames = paste("gamma", 1:garchp, sep = "")
		LB[gpnames] = -10
		UB[gpnames] = 10
		pars[gpnames] = rep(0.5, garchp)
		if(any(substr(start.names, 1, 5) == "gamma")){
			j = which(substr(start.names, 1, 5) == "gamma")
			gpmatch = charmatch(start.names[j], gpnames)
			pars[gpnames[gpmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "gamma")){
			j = which(substr(fixed.names, 1, 5) == "gamma")
			gpmatch = charmatch(fixed.names[j], gpnames)
			pars[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			LB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			UB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	# garchq (beta)
	if(pos[10,3]){
		gqnames = paste("beta", 1:garchq, sep = "")
		LB[gqnames] = -10
		UB[gqnames] = 10
		pars[gqnames] = rep(0.8/garchq,garchq)
		if(any(substr(start.names, 1, 4) == "beta")){
			j = which(substr(start.names, 1, 4) == "beta")
			gqmatch = charmatch(start.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 4) == "beta")){
			j = which(substr(fixed.names, 1, 4) == "beta")
			gqmatch = charmatch(fixed.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			LB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			UB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
		}
	}
	

	# external regressors
	if(pos[11,3]){
		vxnames = paste("vxreg",1:vxn,sep="")
		LB[vxnames] = -200
		UB[vxnames] = 200
		pars[vxnames] = rep(1, vxn)
		if(any(substr(start.names, 1, 5) == "vxreg")){
			j = which(substr(start.names, 1, 5) == "vxreg")
			vxmatch = charmatch(start.names[j],vxnames)
			pars[vxnames[vxmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "vxreg")){
			j = which(substr(fixed.names, 1, 5) == "vxreg")
			vxmatch = charmatch(fixed.names[j],vxnames)
			pars[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
			LB[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
			UB[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	# get the parameter start vector and bounds for the distribution
	dbounds = .DistributionBounds(distribution=dmodel$distribution)
	
	if(pos[12,3]){
		LB["dlambda"] = dbounds$dlambda.LB
		UB["dlambda"] = dbounds$dlambda.UB
		if(is.null(start.pars$dlambda)) pars["dlambda"] = dbounds$dlambda else pars["dlambda"] = start.pars$dlambda[1]
		if(any(substr(fixed.names, 1, 7) == "dlambda")){
			pars["dlambda"] = as.numeric(fixed.pars$dlambda)
			LB["dlambda"] = fixed.pars$dlambda
			UB["dlambda"] = fixed.pars$dlambda
		}
	}
	
	# skew
	if(pos[13,3]){
		LB["skew"] = dbounds$skew.LB
		UB["skew"] = dbounds$skew.UB
		if(is.null(start.pars$skew)) pars["skew"] = dbounds$skew else pars["skew"] = start.pars$skew[1]
		if(any(substr(fixed.names, 1, 4) == "skew")){
			pars["skew"] = as.numeric(fixed.pars$skew)
			LB["skew"] = fixed.pars$skew
			UB["skew"] = fixed.pars$skew
		}
	}
	# shape
	if(pos[14,3]){
		if(is.null(start.pars$shape)) pars["shape"] = dbounds$shape else pars["shape"] = start.pars$shape[1]
		LB["shape"] = dbounds$shape.LB
		UB["shape"] = dbounds$shape.UB
		if(any(substr(fixed.names, 1, 5) == "shape")){
			pars["shape"] = as.numeric(fixed.pars$shape)
			LB["shape"] = fixed.pars$shape
			UB["shape"] = fixed.pars$shape
		}
	}
		
	modelLB = LB
	modelUB = UB
	return(list(pars = pars, modelLB = modelLB, modelUB = modelUB))
}

# gjtGARCH model start parameters
.gjrgarchstart = function(data, model, submodel, garchenv)
{
	# extract required sets
	dscale = get("dscale", garchenv)
	mmodel = get("mmodel", garchenv)
	vmodel = get("vmodel", garchenv)
	dmodel = get("dmodel", garchenv)
	omodel = get("omodel", garchenv)
	start.pars = omodel$start.pars
	opt.pars = omodel$opt.pars
	fixed.pars = omodel$fixed.pars
	pos = omodel$pos.matrix
	meqspars = get("meqspars", garchenv)
	mxn = mmodel$mxn
	vexdata = vmodel$vexdata
	opt.names = names(opt.pars)
	fixed.names = names(fixed.pars)
	start.names = names(start.pars)
	garchp = vmodel$garchOrder[1]
	garchq = vmodel$garchOrder[2]
	vxn = vmodel$vxn
	# get the common to all mean equation parameter start vector
	# and bounds
	meqx = .meqstart(data, meqspars, mmodel, opt.pars, start.pars, fixed.pars, garchenv)
	pars = meqx$opt.pars
	LB = meqx$LB
	UB = meqx$UB
	
	# get the sgarch parameter start vector and bounds
	if(pos[7,3]){
		LB["omega"] = var(data)/100000
		UB["omega"] = var(data)*100000
		if(is.null(start.pars$omega)) pars["omega"] = (var(data, na.rm = TRUE))/1000 else 
			pars["omega"] = start.pars$omega[1]/dscale
		if(any(substr(fixed.names, 1, 5) == "omega")){
			pars["omega"] = as.numeric(fixed.pars$omega)
			LB["omega"] = fixed.pars$omega
			UB["omega"] = fixed.pars$omega
		}
		
	}
	# garchp (alpha)
	# - potentially we could have a problem of the user enters a garch model with
	# p|q>1 and enters a high fixed parameter for either making violating the alpha+beta<1 rule
	# and disrupting the solver...
	
	if(pos[8,3]){
		gpnames = paste("alpha",1:garchp,sep="")
		LB[gpnames] = 0
		UB[gpnames] = 1-TinY
		pars[gpnames] = rep(0.05/garchp,garchp)
		if(any(substr(start.names, 1, 5) == "alpha")){
			j = which(substr(start.names, 1, 5) == "alpha")
			gpmatch = charmatch(start.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "alpha")){
			j = which(substr(fixed.names, 1, 5) == "alpha")
			gpmatch = charmatch(fixed.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			LB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			UB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	# gamma parameter
	if(pos[9,3]){
		gpnames = paste("gamma",1:garchp,sep="")
		LB[gpnames] = -1+TinY
		UB[gpnames] =  1-TinY
		pars[gpnames] = rep(0.05/garchp,garchp)
		if(any(substr(start.names, 1, 5) == "gamma")){
			j = which(substr(start.names, 1, 5) == "gamma")
			gpmatch = charmatch(start.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "gamma")){
			j = which(substr(fixed.names, 1, 5) == "gamma")
			gpmatch = charmatch(fixed.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			LB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			UB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	# garchq (beta)
	if(pos[10,3]){
		gqnames = paste("beta",1:garchq,sep="")
		LB[gqnames] = 0
		UB[gqnames] = 1-TinY
		pars[gqnames] = rep(0.9/garchq,garchq)
		if(any(substr(start.names, 1, 4) == "beta")){
			j = which(substr(start.names, 1, 4) == "beta")
			gqmatch = charmatch(start.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 4) == "beta")){
			j = which(substr(fixed.names, 1, 4) == "beta")
			gqmatch = charmatch(fixed.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			LB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			UB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	# external regressors
	if(pos[11,3]){
		vxnames = paste("vxreg",1:vxn,sep="")
		LB[vxnames] = 0
		UB[vxnames] = 100
		pars[vxnames] = rep(TinY, vxn)
		if(any(substr(start.names, 1, 5) == "vxreg")){
			j = which(substr(start.names, 1, 5) == "vxreg")
			vxmatch = charmatch(start.names[j],vxnames)
			pars[vxnames[vxmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "vxreg")){
			j = which(substr(fixed.names, 1, 5) == "vxreg")
			vxmatch = charmatch(fixed.names[j],vxnames)
			pars[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
			LB[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
			UB[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
		}
	}
	dbounds = .DistributionBounds(distribution=dmodel$distribution)
	
	if(pos[12,3]){
		LB["dlambda"] = dbounds$dlambda.LB
		UB["dlambda"] = dbounds$dlambda.UB
		if(is.null(start.pars$dlambda)) pars["dlambda"] = dbounds$dlambda else pars["dlambda"] = start.pars$dlambda[1]
		if(any(substr(fixed.names, 1, 7) == "dlambda")){
			pars["dlambda"] = as.numeric(fixed.pars$dlambda)
			LB["dlambda"] = fixed.pars$dlambda
			UB["dlambda"] = fixed.pars$dlambda
		}
	}
	
	# skew
	if(pos[13,3]){
		LB["skew"] = dbounds$skew.LB
		UB["skew"] = dbounds$skew.UB
		if(is.null(start.pars$skew)) pars["skew"] = dbounds$skew else pars["skew"] = start.pars$skew[1]
		if(any(substr(fixed.names, 1, 4) == "skew")){
			pars["skew"] = as.numeric(fixed.pars$skew)
			LB["skew"] = fixed.pars$skew
			UB["skew"] = fixed.pars$skew
		}
	}
	# shape
	if(pos[14,3]){
		if(is.null(start.pars$shape)) pars["shape"] = dbounds$shape else pars["shape"] = start.pars$shape[1]
		LB["shape"] = dbounds$shape.LB
		UB["shape"] = dbounds$shape.UB
		if(any(substr(fixed.names, 1, 5) == "shape")){
			pars["shape"] = as.numeric(fixed.pars$shape)
			LB["shape"] = fixed.pars$shape
			UB["shape"] = fixed.pars$shape
		}
	}
	modelLB = LB
	modelUB = UB
	return(list(pars = pars, modelLB = modelLB, modelUB = modelUB))
}

# apARCH model start parameters
.aparchstart = function(data, model, submodel, garchenv)
{
	# extract required sets
	dscale = get("dscale", garchenv)
	mmodel = get("mmodel", garchenv)
	vmodel = get("vmodel", garchenv)
	dmodel = get("dmodel", garchenv)
	omodel = get("omodel", garchenv)
	start.pars = omodel$start.pars
	opt.pars = omodel$opt.pars
	fixed.pars = omodel$fixed.pars
	pos = omodel$pos.matrix
	meqspars = get("meqspars", garchenv)
	mxn = mmodel$mxn
	vexdata = vmodel$vexdata
	opt.names = names(opt.pars)
	fixed.names = names(fixed.pars)
	start.names = names(start.pars)
	garchp = vmodel$garchOrder[1]
	garchq = vmodel$garchOrder[2]
	vxn = vmodel$vxn
	# get the common to all mean equation parameter start vector
	# and bounds
	meqx = .meqstart(data, meqspars, mmodel, opt.pars, start.pars, fixed.pars, garchenv)
	pars = meqx$opt.pars
	LB = meqx$LB
	UB = meqx$UB
	
	# get the sgarch parameter start vector and bounds

	# garchp (alpha)
	# - potentially we could have a problem of the user enters a garch model with
	# p|q>1 and enters a high fixed parameter for either making violating the alpha+beta<1 rule
	# and disrupting the solver...
	if(pos[7,3]){
		LB["omega"] = eps
		UB["omega"] = var(data)*100000
		if(is.null(start.pars$omega)){
			pars["omega"] = (var(data, na.rm = TRUE))/1000 
		} else {
			pars["omega"] = start.pars$omega[1]/dscale
		}
		if(any(substr(fixed.names, 1, 5) == "omega")){
			pars["omega"] = as.numeric(fixed.pars$omega)
			LB["omega"] = fixed.pars$omega
			UB["omega"] = fixed.pars$omega
		}
	}
	
	if(pos[8,3]){
		gpnames = paste("alpha",1:garchp,sep="")
		LB[gpnames] = 0
		UB[gpnames] = 1-TinY
		pars[gpnames] = rep(0.05/garchp,garchp)
		if(any(substr(start.names, 1, 5) == "alpha")){
			j = which(substr(start.names, 1, 5) == "alpha")
			gpmatch = charmatch(start.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "alpha")){
			j = which(substr(fixed.names, 1, 5) == "alpha")
			gpmatch = charmatch(fixed.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			LB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			UB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	# gamma parameter
	if(pos[9,3]){
		gpnames = paste("gamma",1:garchp,sep="")
		LB[gpnames] = 0
		UB[gpnames] = 1-TinY
		pars[gpnames] = rep(0.05/garchp,garchp)
		if(any(substr(start.names, 1, 5) == "gamma")){
			j = which(substr(start.names, 1, 5) == "gamma")
			gpmatch = charmatch(start.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "gamma")){
			j = which(substr(fixed.names, 1, 5) == "gamma")
			gpmatch = charmatch(fixed.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			LB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			UB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	if(pos[10,3]){
		LB["delta"] = 0.01
		UB["delta"] = 3.5
		if(is.null(start.pars$delta)) pars["delta"] = 2 else pars["delta"] = start.pars$delta[1]
		if(any(substr(fixed.names, 1, 5) == "delta")){
			pars["delta"] = as.numeric(fixed.pars$delta)
			LB["delta"] = fixed.pars$delta
			UB["delta"] = fixed.pars$delta
		}
		
	}

	# garchq (beta)
	if(pos[11,3]){
		gqnames = paste("beta",1:garchq,sep="")
		LB[gqnames] = 0
		UB[gqnames] = 1-TinY
		pars[gqnames] = rep(0.9/garchq,garchq)
		if(any(substr(start.names, 1, 4) == "beta")){
			j = which(substr(start.names, 1, 4) == "beta")
			gqmatch = charmatch(start.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 4) == "beta")){
			j = which(substr(fixed.names, 1, 4) == "beta")
			gqmatch = charmatch(fixed.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			LB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			UB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	# external regressors
	if(pos[12,3]){
		vxnames = paste("vxreg",1:vxn,sep="")
		LB[vxnames] = 0
		UB[vxnames] = 100
		pars[vxnames] = rep(TinY, vxn)
		if(any(substr(start.names, 1, 5) == "vxreg")){
			j = which(substr(start.names, 1, 5) == "vxreg")
			vxmatch = charmatch(start.names[j],vxnames)
			pars[vxnames[vxmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "vxreg")){
			j = which(substr(fixed.names, 1, 5) == "vxreg")
			vxmatch = charmatch(fixed.names[j],vxnames)
			pars[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
			LB[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
			UB[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
		}
	}
	dbounds = .DistributionBounds(distribution=dmodel$distribution)
	
	if(pos[13,3]){
		LB["dlambda"] = dbounds$dlambda.LB
		UB["dlambda"] = dbounds$dlambda.UB
		if(is.null(start.pars$dlambda)) pars["dlambda"] = dbounds$dlambda else pars["dlambda"] = start.pars$dlambda[1]
		if(any(substr(fixed.names, 1, 7) == "dlambda")){
			pars["dlambda"] = as.numeric(fixed.pars$dlambda)
			LB["dlambda"] = fixed.pars$dlambda
			UB["dlambda"] = fixed.pars$dlambda
		}
	}
	# skew
	if(pos[14,3]){
		LB["skew"] = dbounds$skew.LB
		UB["skew"] = dbounds$skew.UB
		if(is.null(start.pars$skew)) pars["skew"] = dbounds$skew else pars["skew"] = start.pars$skew[1]
		if(any(substr(fixed.names, 1, 4) == "skew")){
			pars["skew"] = as.numeric(fixed.pars$skew)
			LB["skew"] = fixed.pars$skew
			UB["skew"] = fixed.pars$skew
		}
	}
	# shape
	if(pos[15,3]){
		if(is.null(start.pars$shape)) pars["shape"] = dbounds$shape else pars["shape"] = start.pars$shape[1]
		LB["shape"] = dbounds$shape.LB
		UB["shape"] = dbounds$shape.UB
		if(any(substr(fixed.names, 1, 5) == "shape")){
			pars["shape"] = as.numeric(fixed.pars$shape)
			LB["shape"] = fixed.pars$shape
			UB["shape"] = fixed.pars$shape
		}
	}
	modelLB = LB
	modelUB = UB
	return(list(pars = pars, modelLB = modelLB, modelUB = modelUB))
}

# sGARCH model start parameters
.igarchstart = function(data, model, submodel, garchenv)
{
	# extract required sets
	dscale = get("dscale", garchenv)
	mmodel = get("mmodel", garchenv)
	vmodel = get("vmodel", garchenv)
	dmodel = get("dmodel", garchenv)
	omodel = get("omodel", garchenv)
	start.pars = omodel$start.pars
	opt.pars = omodel$opt.pars
	fixed.pars = omodel$fixed.pars
	pos = omodel$pos.matrix
	meqspars = get("meqspars", garchenv)
	mxn = mmodel$mxn
	vexdata = vmodel$vexdata
	opt.names = names(opt.pars)
	fixed.names = names(fixed.pars)
	start.names = names(start.pars)
	garchp = vmodel$garchOrder[1]
	garchq = vmodel$garchOrder[2]
	vxn = vmodel$vxn
	# get the common to all mean equation parameter start vector
	# and bounds
	meqx = .meqstart(data, meqspars, mmodel, opt.pars, start.pars, fixed.pars, garchenv)
	pars = meqx$opt.pars
	LB = meqx$LB
	UB = meqx$UB
	
	# get the sgarch parameter start vector and bounds
	# get the sgarch parameter start vector and bounds
	if(pos[7,3]){
		LB["omega"] = var(data)/100000
		UB["omega"] = var(data)*100000
		if(is.null(start.pars$omega)) pars["omega"] = (var(data, na.rm = TRUE))/1000 else 
			pars["omega"] = start.pars$omega[1]/dscale
		if(any(substr(fixed.names, 1, 5) == "omega")){
			pars["omega"] = as.numeric(fixed.pars$omega)
			LB["omega"] = fixed.pars$omega
			UB["omega"] = fixed.pars$omega
		}
		
	}
	# garchp (alpha)
	# - potentially we could have a problem of the user enters a garch model with
	# p|q>1 and enters a high fixed parameter for either making violating the alpha+beta<1 rule
	# and disrupting the solver...
	
	if(pos[8,3]){
		gpnames = paste("alpha",1:garchp,sep="")
		LB[gpnames] = 0
		UB[gpnames] = 1-TinY
		pars[gpnames] = rep(0.05/garchp,garchp)
		if(any(substr(start.names, 1, 5) == "alpha")){
			j = which(substr(start.names, 1, 5) == "alpha")
			gpmatch = charmatch(start.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "alpha")){
			j = which(substr(fixed.names, 1, 5) == "alpha")
			gpmatch = charmatch(fixed.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			LB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			UB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	# garchq (beta)
	if(pos[9,3]){
		gqnames = paste("beta",1:garchq,sep="")
		LB[gqnames] = 0
		UB[gqnames] = c(rep(1-TinY,garchq-1),0)
		pars[gqnames] = rep(0.9/garchq,garchq)
		if(any(substr(start.names, 1, 4) == "beta")){
			j = which(substr(start.names, 1, 4) == "beta")
			gqmatch = charmatch(start.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 4) == "beta")){
			j = which(substr(fixed.names, 1, 4) == "beta")
			gqmatch = charmatch(fixed.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			LB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			UB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			# ensure the last beta is 0
			if(garchq==1){
				LB[paste("beta1")]=0
				UB[paste("beta1")]=0
			} else{
				LB[paste("beta",garchq,sep="")]=0
				UB[paste("beta",garchq,sep="")]=0
			}
		}
	}
	
	# external regressors
	if(pos[10,3]){
		vxnames = paste("vxreg",1:vxn,sep="")
		LB[vxnames] = 0
		UB[vxnames] = 100
		pars[vxnames] = rep(TinY, vxn)
		if(any(substr(start.names, 1, 5) == "vxreg")){
			j = which(substr(start.names, 1, 5) == "vxreg")
			vxmatch = charmatch(start.names[j],vxnames)
			pars[vxnames[vxmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "vxreg")){
			j = which(substr(fixed.names, 1, 5) == "vxreg")
			vxmatch = charmatch(fixed.names[j],vxnames)
			pars[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
			LB[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
			UB[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
		}
	}
	dbounds = .DistributionBounds(distribution=dmodel$distribution)
	
	if(pos[11,3]){
		LB["dlambda"] = dbounds$dlambda.LB
		UB["dlambda"] = dbounds$dlambda.UB
		if(is.null(start.pars$dlambda)) pars["dlambda"] = dbounds$dlambda else pars["dlambda"] = start.pars$dlambda[1]
		if(any(substr(fixed.names, 1, 7) == "dlambda")){
			pars["dlambda"] = as.numeric(fixed.pars$dlambda)
			LB["dlambda"] = fixed.pars$dlambda
			UB["dlambda"] = fixed.pars$dlambda
		}
	}
	
	# skew
	if(pos[12,3]){
		LB["skew"] = dbounds$skew.LB
		UB["skew"] = dbounds$skew.UB
		if(is.null(start.pars$skew)) pars["skew"] = dbounds$skew else pars["skew"] = start.pars$skew[1]
		if(any(substr(fixed.names, 1, 4) == "skew")){
			pars["skew"] = as.numeric(fixed.pars$skew)
			LB["skew"] = fixed.pars$skew
			UB["skew"] = fixed.pars$skew
		}
	}
	# shape
	if(pos[13,3]){
		if(is.null(start.pars$shape)) pars["shape"] = dbounds$shape else pars["shape"] = start.pars$shape[1]
		LB["shape"] = dbounds$shape.LB
		UB["shape"] = dbounds$shape.UB
		if(any(substr(fixed.names, 1, 5) == "shape")){
			pars["shape"] = as.numeric(fixed.pars$shape)
			LB["shape"] = fixed.pars$shape
			UB["shape"] = fixed.pars$shape
		}
	}
	modelLB = LB
	modelUB = UB
	return(list(pars = pars, modelLB = modelLB, modelUB = modelUB))
}

# anstGARCH model start parameters
.anstgarchstart = function(data, model, submodel, garchenv)
{
	# extract required sets
	dscale = get("dscale", garchenv)
	mmodel = get("mmodel", garchenv)
	vmodel = get("vmodel", garchenv)
	dmodel = get("dmodel", garchenv)
	omodel = get("omodel", garchenv)
	start.pars = omodel$start.pars
	opt.pars = omodel$opt.pars
	fixed.pars = omodel$fixed.pars
	pos = omodel$pos.matrix
	meqspars = get("meqspars", garchenv)
	mxn = mmodel$mxn
	vexdata = vmodel$vexdata
	opt.names = names(opt.pars)
	fixed.names = names(fixed.pars)
	start.names = names(start.pars)
	garchp1 = vmodel$garchOrder[1]
	garchq1 = vmodel$garchOrder[2]
	
	garchp2 = vmodel$garchOrder[3]
	garchq2 = vmodel$garchOrder[4]
	
	vxn = vmodel$vxn
	# get the common to all mean equation parameter start vector
	# and bounds
	meqx = .meqstart(data, meqspars, mmodel, opt.pars, start.pars, fixed.pars, garchenv)
	pars = meqx$opt.pars
	LB = meqx$LB
	UB = meqx$UB
	
	# get the sgarch parameter start vector and bounds
	if(pos[7,3]){
		LB["omega1"] = var(data)/100000/dscale
		UB["omega1"] = var(data)*100000/dscale
		
		if(is.null(start.pars$omega1)) pars["omega1"] = (var(data, na.rm = TRUE))/dscale else pars["omega1"] = start.pars$omega1/dscale
				
		if(any(substr(fixed.names, 1, 6) == "omega1")){
			pars["omega1"] = as.numeric(fixed.pars$omega1)
			LB["omega1"] = fixed.pars$omega1
			UB["omega1"] = fixed.pars$omega1
		}
	}
	
	if(pos[8,3]){
		gpnames = paste("alpha1", 1:garchp1, sep = "")
		LB[gpnames] = eps
		UB[gpnames] = 1-TinY
		pars[gpnames] = rep(0.05/garchp1, garchp1)
		if(any(substr(start.names, 1, 6) == "alpha1")){
			j = which(substr(start.names, 1, 6) == "alpha1")
			gpmatch = charmatch(start.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 6) == "alpha1")){
			j = which(substr(fixed.names, 1, 6) == "alpha1")
			gpmatch = charmatch(fixed.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			LB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			UB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	# garchq (beta)
	if(pos[9,3]){
		gqnames = paste("beta1", 1:garchq1, sep = "")
		LB[gqnames] = eps
		UB[gqnames] = 1-TinY
		pars[gqnames] = rep(0.5/garchq1, garchq1)
		if(any(substr(start.names, 1, 5) == "beta1")){
			j = which(substr(start.names, 1, 5) == "beta1")
			gqmatch = charmatch(start.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "beta1")){
			j = which(substr(fixed.names, 1, 5) == "beta1")
			gqmatch = charmatch(fixed.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			LB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			UB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	if(pos[10,3]){
		LB["omega2"] = -var(data)*100000/dscale
		UB["omega2"] = var(data)*100000/dscale
		
		if(is.null(start.pars$omega2)) pars["omega2"] = var(data)/dscale else pars["omega2"] = start.pars$omega2/dscale
		
		if(any(substr(fixed.names, 1, 6) == "omega2")){
			pars["omega2"] = as.numeric(fixed.pars$omega2)
			LB["omega2"] = fixed.pars$omega2
			UB["omega2"] = fixed.pars$omega2
		}
	}
	
	if(pos[11,3]){
		gpnames = paste("alpha2", 1:garchp2, sep = "")
		LB[gpnames] = -1 + TinY
		UB[gpnames] = 1-TinY
		pars[gpnames] = rep(0.05/garchp2, garchp2)
		if(any(substr(start.names, 1, 6) == "alpha2")){
			j = which(substr(start.names, 1, 6) == "alpha2")
			gpmatch = charmatch(start.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 6) == "alpha2")){
			j = which(substr(fixed.names, 1, 6) == "alpha2")
			gpmatch = charmatch(fixed.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			LB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			UB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
		}
	}
	if(pos[12,3]){
		gqnames = paste("beta2", 1:garchq2, sep = "")
		LB[gqnames] = -1 + TinY
		UB[gqnames] = 1-TinY
		pars[gqnames] = rep(0.5/garchq2, garchq2)
		if(any(substr(start.names, 1, 5) == "beta2")){
			j = which(substr(start.names, 1, 5) == "beta2")
			gqmatch = charmatch(start.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "beta2")){
			j = which(substr(fixed.names, 1, 5) == "beta2")
			gqmatch = charmatch(fixed.names[j],gqnames)
			pars[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			LB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
			UB[gqnames[gqmatch]] = as.numeric(fixed.pars[j])
		}
	}
	
	# gamma parameter
	if(pos[13,3]){
		gpnames = "gamma"
		LB[gpnames] = eps
		UB[gpnames] = 100
		pars[gpnames] = 1
		if(any(substr(start.names, 1, 5) == "gamma")){
			j = which(substr(start.names, 1, 5) == "gamma")
			gpmatch = charmatch(start.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "gamma")){
			j = which(substr(fixed.names, 1, 5) == "gamma")
			gpmatch = charmatch(fixed.names[j],gpnames)
			pars[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			LB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
			UB[gpnames[gpmatch]] = as.numeric(fixed.pars[j])
		}
	}
	

	
	# external regressors
	if(pos[14,3]){
		vxnames = paste("vxreg",1:vxn,sep="")
		LB[vxnames] = 0
		UB[vxnames] = 100
		pars[vxnames] = rep(TinY, vxn)
		if(any(substr(start.names, 1, 5) == "vxreg")){
			j = which(substr(start.names, 1, 5) == "vxreg")
			vxmatch = charmatch(start.names[j],vxnames)
			pars[vxnames[vxmatch]] = as.numeric(start.pars[j])
		}
		if(any(substr(fixed.names, 1, 5) == "vxreg")){
			j = which(substr(fixed.names, 1, 5) == "vxreg")
			vxmatch = charmatch(fixed.names[j],vxnames)
			pars[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
			LB[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
			UB[vxnames[vxmatch]] = as.numeric(fixed.pars[j])
		}
	}
	dbounds = .DistributionBounds(distribution=dmodel$distribution)
	
	if(pos[15,3]){
		LB["dlambda"] = dbounds$dlambda.LB
		UB["dlambda"] = dbounds$dlambda.UB
		if(is.null(start.pars$dlambda)) pars["dlambda"] = dbounds$dlambda else pars["dlambda"] = start.pars$dlambda[1]
		if(any(substr(fixed.names, 1, 7) == "dlambda")){
			pars["dlambda"] = as.numeric(fixed.pars$dlambda)
			LB["dlambda"] = fixed.pars$dlambda
			UB["dlambda"] = fixed.pars$dlambda
		}
	}
	
	# skew
	if(pos[16,3]){
		LB["skew"] = dbounds$skew.LB
		UB["skew"] = dbounds$skew.UB
		if(is.null(start.pars$skew)) pars["skew"] = dbounds$skew else pars["skew"] = start.pars$skew[1]
		if(any(substr(fixed.names, 1, 4) == "skew")){
			pars["skew"] = as.numeric(fixed.pars$skew)
			LB["skew"] = fixed.pars$skew
			UB["skew"] = fixed.pars$skew
		}
	}
	# shape
	if(pos[17,3]){
		if(is.null(start.pars$shape)) pars["shape"] = dbounds$shape else pars["shape"] = start.pars$shape[1]
		LB["shape"] = dbounds$shape.LB
		UB["shape"] = dbounds$shape.UB
		if(any(substr(fixed.names, 1, 5) == "shape")){
			pars["shape"] = as.numeric(fixed.pars$shape)
			LB["shape"] = fixed.pars$shape
			UB["shape"] = fixed.pars$shape
		}
	}
	modelLB = LB
	modelUB = UB
	return(list(pars = pars, modelLB = modelLB, modelUB = modelUB))
}

# we modify the rsFit method from the fArma package:
# r/s fit for darfima parameter
.rsfit = function(x, levels = 50, minnpts = 3, cut.off = 10^c(0.7, 2.5))
{  
	# A functions implemented by Diethelm Wuertz
	
	# Description:
	#   R/S Statistic Method - [Taqqu 3.6]
	
	# Arguments:
	#   x - a numeric vector, a 'timeSeries' object, or any other
	#       object which can be transformed into a vector by the
	#       function 'as.vector'.
	#   levels - the number of aggregation levels
	#   minnpts - the minimum block size
	#   cut.off - the lower and upper cut off for the fit
	
	# FUNCTION:
	
	# Settings:
	call = match.call()
	data = list(x = x)
	x = as.vector(x)
	n = length(x)
	increment = (log10(n/minnpts))/levels
	M = floor(10^((1:levels)*increment))
	M = M[M > 1]
	
	# R/S Method:
	Y = cumsum(x)
	Y2 = cumsum(x*x)
	RS = NULL
	for (m in M) {
		S = sqrt(Y2[m]/m - (Y[m]/m)^2)
		Z = Y[1:m]-(1:m)*Y[m]/m
		STATS = (max(Z) - min(Z))/S
		RS = c(RS, STATS)
	}
	# Fit:
	wt = trunc((sign((M-cut.off[1])*(cut.off[2]-M))+1)/2)
	fit = lsfit(log10(M), log10(RS), wt)
	fitH = fit
	fitH$wt = NULL
	diag = as.data.frame(ls.print(fitH, print.it = FALSE)[[2]][[1]])
	beta = fit$coef[[2]]
	return(beta)	
}
