\name{cgarchspec-methods}
\docType{methods}
\alias{cgarchspec}
\alias{cgarchspec,ANY-method}
\alias{cgarchspec,uGARCHmultispec-method}
\title{function: Copula-GARCH Specification}
\description{
Method for creating a Copula-GARCH specification object prior to fitting.
}
\usage{
cgarchspec(uspec, VAR = FALSE, VAR.opt = list(robust = FALSE, lag = 1, lag.max = NULL, 
lag.criterion = c("AIC", "HQ", "SC", "FPE"), external.regressors = NULL, 
robust.control = list("gamma" = 0.25, "delta" = 0.01, "nc" = 10, "ns" = 500)), 
dccOrder = c(1,1), distribution.model = list(copula = c("mvnorm", "mvt"), method = c("Kendall", "ML"),
time.varying = FALSE, transformation = c("parametric", "empirical", "spd")), start.pars = list(), 
fixed.pars = list())
}
\arguments{
  \item{uspec}{
  A \code{\linkS4class{uGARCHmultispec}} object created by calling \sQuote{multispec} on
  a list of univariate GARCH specifications.}
  \item{VAR}{ Whether to fit a VAR model to the data.}
  \item{VAR.opt}{ The VAR model options.}
  \item{dccOrder}{ The DCC autoregressive order.}
  \item{distribution.model}{ The Copula distribution model. Currently the multivariate Normal and
  Student Copula are supported. The method is for the static Copula. }
  \item{time.varying}{ Whether to fit a dynamic DCC Copula. }
  \item{transformation}{ The type of transformation to apply to the marginal innovations of the GARCH fitted models. 
  Supported methods are parametric (Inference Function of Margins), empirical (Pseudo ML), and Semi-Parametric using
  a kernel interior and GPD tails (via the spd package).}
  \item{start.pars}{ (optional) Starting values for the DCC parameters (starting values for the 
  univariate garch specification should be passed directly via the \sQuote{uspec} object).}
  \item{fixed.pars}{ (optional) Fixed DCC parameters.}
}
\value{
A \code{\linkS4class{cGARCHspec}} object containing details of the Copula-GARCH specification.
}
\details{
The transformation method allows for parametric (Inference-Functions for Margins), 
empirical (Pseudo-Likelihood) and semi-parametric (via the spd package).\cr
When the Student Copula is jointly estimated with student margins having so that a common shape
parameter is obtained, this results in the multivariate Student distribution. When estimating the
Student Copula with disparate margins, a meta-student distribution is obtained. Additionally, the
correlation parameter in the Student Copula may be estimated either by Kendall's tau transformation
or Maximum Likelihood.\cr
The \sQuote{VAR.opt} allows for a robust version of VAR based on the multivariate Least Trimmed Squares Estimator 
described in Croux and Joossens (2008). The \sQuote{robust.control} includes additional tuning parameters to the
robust regression including the proportion to trim (\dQuote{gamma}), the critical value for Reweighted estimator
(\dQuote{delta}), the number of subsets (\dQuote{ns}) and the number of C-steps (\dQuote{nc}).
}
\author{Alexios Ghalanos}
\examples{
# same specification for all:
uspec = multispec( replicate(4, ugarchspec()) )
# different univariate specifications
uspec = multispec( c( ugarchspec(variance.model = list(model = "sGARCH")), 
ugarchspec(variance.model = list(model = "gjrGARCH")), ugarchspec(mean.model = list(armaOrder = c(2,1))) ) )
# pass uspec into dccspec
spec = cgarchspec(uspec, dccOrder = c(1,1))
spec
}
\keyword{methods}