\name{fpm-methods}
\docType{methods}
\alias{fpm-methods}
\alias{fpm}
\alias{fpm,ANY-method}
\alias{fpm,uGARCHforecast-method}
\alias{fpm,ARFIMAforecast-method}
\title{function: Univariate GARCH and ARFIMA Forecast Performance Measures Method}
\description{
A method for generating forecast performance measures from a forecasted object
of class \code{\linkS4class{uGARCHforecast}} or \code{\linkS4class{ARFIMAforecast}} 
subject to either out of sample data being available from the fitted routine or user 
provided realized data.
}
\usage{
fpm(object, ...)
}
\arguments{
\item{object}{A \code{\linkS4class{uGARCHforecast}} or \code{\linkS4class{ARFIMAforecast}} object}
\item{...}{ Additional arguments include \code{realized} which may me a matrix (or vector) two (one)
named columns, with the names being \dQuote{series} and \dQuote{sigma} (not for ARFIMA case) indicating 
the realized data which will be used for the loss functions. These must be at least as long as the 
rolling forecast horizon. Additionally and required if the \code{realized} is used, 
the \code{realized.type} option indicates the type of realized data provided and is either \dQuote{both}, 
\dQuote{sigma} or \dQuote{series} (for ARFIMA this is not applicable since only series is available).}
}
\value{
A \code{\linkS4class{uGARCHfpm}} or code{\linkS4class{ARFIMAfpm}} object containing details of the 
forecast performance measures for which extractor and show methods exist (see class for details).
}
\details{
The method is only valid for objects which have been previously called with the \code{out.sample}
option in the fitting routine with a minimum of 10 out of sample point available. The \code{realized}
option, while still requiring the \code{out.sample} to have been used, will allow either a matrix with
both the realized sigma and data or a vector with either to be provided for the calculation of the
loss measures. The \code{realized.type} indicates what realized data is provided, and in the case
of \dQuote{both} and a matrix, unless the matrix columns are named (with valid names being
\dQuote{sigma} and \dQuote{series}, the function will assume that sigma is the first column and
the series is the second one.
}
\section{Methods}{
\describe{
\item{object = "ANY"}{ Generic method for expansion to other classes }
\item{object = "uGARCHforecast"}{ The method currently supports this class. }
\item{object = "ARFIMAforecast"}{ The method currently supports this class. }
}}
\examples{
\dontrun{
data(dmbp)
spec = ugarchspec(variance.model = list(model = "sGARCH", garchOrder = c(1,1)), 
		mean.model = list(armaOrder = c(1,1), include.mean = TRUE), distribution.model = "std")
fit = ugarchfit(data = dmbp[,1], out.sample = 50, spec = spec)
pred = ugarchforecast(fit, n.ahead = 50)
pred.fpm = fpm(pred)
# show method
pred.fpm
# extract meanloss
as.data.frame(pred.fpm, which = "sigma", type = "meanloss")
# extract the actual loss series
as.data.frame(pred.fpm, which = "sigma", type = "loss", rollframe = 0)

# an example with rolling:
pred = ugarchforecast(fit, n.ahead = 50, n.roll = 2)
pred.fpm = fpm(pred)
# show method
pred.fpm
# extract meanloss
as.data.frame(pred.fpm, which = "sigma", type = "meanloss")
# extract the actual loss series (notice the NA's at the end since
# we are calculating on 50 + 2 (roll) points using only 50 out.sample data points (for realized)
as.data.frame(pred.fpm, which = "sigma", type = "loss", rollframe = 2)

# an example with fictional realized:
pred = ugarchforecast(fit, n.ahead = 50, n.roll = 1)
pred.fpm = fpm(pred, realized = rep(-0.01, 52), realized.type = "series")
# show method
pred.fpm
# extract meanloss
as.data.frame(pred.fpm, which = "sigma", type = "meanloss")
# extract the actual loss series (notice the NA's at the end since
# we are calculating on 50 + 2 (roll) points using only 50 out.sample data points (for realized)
as.data.frame(pred.fpm, which = "series", type = "loss", rollframe = 1)
}}
\keyword{methods}
