\name{goGARCHforecast-class}
\docType{class}
\alias{goGARCHforecast-class}
\alias{convolution,goGARCHforecast-method}
\alias{gportmoments,goGARCHforecast-method}
\alias{rcokurt,goGARCHforecast-method}
\alias{rcor,goGARCHforecast-method}
\alias{rcoskew,goGARCHforecast-method}
\alias{rcov,goGARCHforecast-method}
\alias{show,goGARCHforecast-method}
\title{class: GO-GARCH Forecast Class}
\description{Class for the GO-GARCH forecast.}
\section{Objects from the Class}{
The class is returned by calling the function \code{\link{gogarchforecast}}.
}
\section{Slots}{
  \describe{
    \item{\code{mforecast}:}{Multivariate forecast object.}
    \item{\code{uforecast}:}{Univariate forecast of class \code{"uGARCHmultiforecast"}.}
  }
}
\section{Extends}{
Class \code{"\linkS4class{mGARCHforecast}"}, directly.
Class \code{"\linkS4class{GARCHforecast}"}, by class "mGARCHforecast", distance 2.
Class \code{"\linkS4class{rGARCH}"}, by class "mGARCHforecast", distance 3.
}
\section{Methods}{
  \describe{
    \item{convolution}{\code{signature(object = "goGARCHforecast")}:\cr
    function:\cr
    \bold{convolution(object, weights, fft.step = 0.01, fft.by = 0.0001, fft.support = c(-1, 1), 
		support.method = c("user", "adaptive"), use.ff = TRUE, parallel = FALSE, 
		parallel.control = list(pkg = c("multicore", "snowfall"), cores = 2), trace = 0,...)}\cr
    The convolution method takes a goGARCHforecast object and a weights vector or matrix and calculates the weighted
    density. If a vector is given, it must be the same length as the number of assets, otherwise a matrix with
    row dimension equal to the row dimension of the filtered dataset (i.e. less any lags).
    In the case of the multivariate normal distribution, this simply returns the linear and quadratic transformation 
    of the mean and covariance matrix, while in the multivariate affine NIG distribution this is based on the 
    numerical inversion by FFT of the characteristic function. In that case, the \dQuote{fft.step} option determines
    the stepsize for tuning the characteristic function inversion, \dQuote{fft.by} determines the resolution for 
    the equally spaced support given by \dQuote{fft.support}, while the use of the \dQuote{ff} package is recommended 
    to avoid memory problems on some systems and is turned on via the \dQuote{use.ff} option. The \dQuote{support.method} 
    option allows either a fixed support range to be given (option \sQuote{user}), else an adaptive method is used based on the 
    min and max of the assets at each point in time at the 0.00001 and 1-0.00001 quantiles. The range is equally spaced subject 
    to the \dQuote{fft.by} value but the returned object no longer makes of the \dQuote{ff} package returning instead a list. 
    Finally, the option for parallel computation is available, though it is far more efficient to use (in unix based systems only) 
    using the \dQuote{multicore} package than \dQuote{snowfall}.}
    \item{gportmoments}{\code{signature(object = "goGARCHforecast")}:\cr
    function:\cr
    \bold{gportmoments(object, weights)}\cr
    Calculates the first 4 portfolio moments using the geometric properties of the model, given a
    matrix of asset weights with row dimension equal to the row dimension of the filtered 
    dataset (i.e. less any lags). The kurtosis is not returned for systems of dimensions greater 
    than 11 because of the memory issues related to calculations on the co-kurtosis tensor.
    }
    \item{rcokurt}{\code{signature(object = "goGARCHforecast")}:\cr
    function:\cr
    \bold{rcokurt(object, from = 1, to = 1)}\cr
    Returns the 'time-varying'  NxN^4 cokurtosis tensor in array format and is limited to a dimensions of 11 assets.
    The \dQuote{from} and \dQuote{to} options indicate the time indices for which to return the arrays. Because of
    memory issues, this is limited to 100 indices.}
    \item{rcoskew}{\code{signature(object = "goGARCHforecast")}:\cr
    function:\cr
    \bold{rcoskew(object, from = 1, to = 1)}\cr
    Returns the 'time-varying'  NxN^3 coskewness tensor in array format. The \dQuote{from} and \dQuote{to} 
    options indicate the time indices for which to return the arrays. Because of memory issues, 
    this is limited to 100 indices.}
    \item{rcov}{\code{signature(object = "goGARCHforecast")}: Returns the time-varying NxN covariance matrix in 
    array format.}
    \item{rcor}{\code{signature(object = "goGARCHforecast")}: Returns the time-varying NxN correlation matrix in 
    array format. }
    \item{show}{\code{signature(object = "goGARCHforecast")}: Summary method for the forecast object. }
	}
}
\references{
de Athayde, G.M. and Flores Jr, R.G. \emph{On Certain Geometric Aspects of Portfolio Optimisation with Higher Moments},
2002, mimeo.\cr
Paolella, M.S.\emph{Intermediate Probability - A Computational Approach}, 2007, Wiley-Interscience.\cr
}
\author{Alexios Ghalanos}
\note{
The reference by Paolella (2007) contains more details on the algorithm for the characteristic function inversion
via FFT. The de Athayde and Flores (2002) paper is the basis for the geometric properties of the higher moment
tensors.\cr
Forecasts are carried out on the time varying parameters of the factor distributions, and then scaled and transformed
to those of the assets after adding back the mean forecast (which is either a constant or the VAR mean forecast).
}
\keyword{classes}
