\name{goGARCHroll-class}
\docType{class}
\alias{goGARCHroll-class}
\alias{rcor,goGARCHroll-method}
\alias{rcov,goGARCHroll-method}
\alias{convolution,goGARCHroll-method}
\title{class: GO-GARCH Roll Class}
\description{Class for the GO-GARCH Roll.}
\section{Objects from the Class}{
The class is returned by calling the function \code{\link{gogarchroll}}.
}
\section{Slots}{
  \describe{
    \item{\code{forecast}:}{Object of class \code{"vector"} which contains the rolling forecasts of the
    distributional parameters for each factor.}
    \item{\code{spec}:}{Object of class \code{"goGARCHspec"}.}
  }
}
\section{Extends}{
Class \code{"\linkS4class{mGARCHroll}"}, directly.
Class \code{"\linkS4class{GARCHroll}"}, by class "mGARCHroll", distance 2.
Class \code{"\linkS4class{rGARCH}"}, by class "mGARCHroll", distance 3.
}
\section{Methods}{
\describe{  
    \item{rcov}{\code{signature(object = "goGARCHroll")}: Returns the time-varying NxN covariance matrix in 
    array format.}
    \item{rcor}{\code{signature(object = "goGARCHroll")}: Returns the time-varying NxN correlation matrix in 
    array format.}
    \item{convolution}{\code{signature(object = "goGARCHroll")}:\cr
    function:\cr
    \bold{convolution(object, weights, fft.step = 0.01, fft.by = 0.0001, fft.support = c(-1, 1), 
		support.method = c("user", "adaptive"), use.ff = TRUE, parallel = FALSE, 
		parallel.control = list(pkg = c("multicore", "snowfall"), cores = 2), trace = 0,...)}\cr
    The convolution method takes a goGARCHroll object and a weights vector or matrix and calculates the weighted
    density. If a vector is given, it must be the same length as the number of assets, otherwise a matrix with
    row dimension equal to the row dimension of the filtered dataset (i.e. less any lags).
    In the case of the multivariate normal distribution, this simply returns the linear and quadratic transformation 
    of the mean and covariance matrix, while in the multivariate affine NIG distribution this is based on the 
    numerical inversion by FFT of the characteristic function. In that case, the \dQuote{fft.step} option determines
    the stepsize for tuning the characteristic function inversion, \dQuote{fft.by} determines the resolution for 
    the equally spaced support given by \dQuote{fft.support}, while the use of the \dQuote{ff} package is recommended 
    to avoid memory problems on some systems and is turned on via the \dQuote{use.ff} option. The \dQuote{support.method} 
    option allows either a fixed support range to be given (option \sQuote{user}), else an adaptive method is used based on the 
    min and max of the assets at each point in time at the 0.00001 and 1-0.00001 quantiles. The range is equally spaced subject 
    to the \dQuote{fft.by} value but the returned object no longer makes of the \dQuote{ff} package returning instead a list. 
    Finally, the option for parallel computation is available, though it is far more efficient to use (in unix based systems only) 
    using the \dQuote{multicore} package than \dQuote{snowfall}.}
   }
}
\author{Alexios Ghalanos}
\keyword{classes}
