\name{goGARCHsim-class}
\docType{class}
\alias{goGARCHsim-class}
\alias{convolution,goGARCHsim-method}
\alias{gportmoments,goGARCHsim-method}
\alias{rcokurt,goGARCHsim-method}
\alias{rcor,goGARCHsim-method}
\alias{rcoskew,goGARCHsim-method}
\alias{rcov,goGARCHsim-method}
\title{class: GO-GARCH Simultion Class}
\description{Class for the GO-GARCH Simulation.}
\section{Objects from the Class}{
The class is returned by calling the function \code{\link{gogarchsim}}.
}
\section{Slots}{
  \describe{
    \item{\code{msim}:}{Object of class \code{"vector"} The multivariate simulation list.}
    \item{\code{usim}:}{Object of class \code{"vector"} The univariate simulation list.}
  }
}
\section{Extends}{
Class \code{"\linkS4class{mGARCHsim}"}, directly.
Class \code{"\linkS4class{GARCHsim}"}, by class "mGARCHsim", distance 2.
Class \code{"\linkS4class{rGARCH}"}, by class "mGARCHsim", distance 3.
}
\section{Methods}{
  \describe{
    \item{convolution}{\code{signature(object = "goGARCHsim")}:\cr
    function:\cr
        \bold{convolution(object, weights, fft.step = 0.01, fft.by = 0.0001, fft.support = c(-1, 1), 
		support.method = c("user", "adaptive"), use.ff = TRUE, sim = 1, parallel = FALSE, 
		parallel.control = list(pkg = c("multicore", "snowfall"), cores = 2), trace = 0,...)}\cr
    The convolution method takes a goGARCHsim object and a weights vector or matrix and calculates the weighted
    density. If a vector is given, it must be the same length as the number of assets, otherwise a matrix with
    row dimension equal to the row dimension of the filtered dataset (i.e. less any lags).
    The \dQuote{sim} option indicates the simulation index to  use, given the \dQuote{m.sim} option chosen in 
    the call to the simulation function.
    In the case of the multivariate normal distribution, this simply returns the linear and quadratic transformation 
    of the mean and covariance matrix, while in the multivariate affine NIG distribution this is based on the 
    numerical inversion by FFT of the characteristic function. In that case, the \dQuote{fft.step} option determines
    the stepsize for tuning the characteristic function inversion, \dQuote{fft.by} determines the resolution for 
    the equally spaced support given by \dQuote{fft.support}, while the use of the \dQuote{ff} package is recommended 
    to avoid memory problems on some systems and is turned on via the \dQuote{use.ff} option. The \dQuote{support.method} 
    option allows either a fixed support range to be given (option \sQuote{user}), else an adaptive method is used based on the 
    min and max of the assets at each point in time at the 0.00001 and 1-0.00001 quantiles. The range is equally spaced subject 
    to the \dQuote{fft.by} value but the returned object no longer makes of the \dQuote{ff} package returning instead a list. 
    Finally, the option for parallel computation is available, though it is far more efficient to use (in unix based systems only) 
    using the \dQuote{multicore} package than \dQuote{snowfall}.}
    \item{gportmoments}{\code{signature(object = "goGARCHsim")}:\cr
     function:\cr
    \bold{gportmoments(object, weights, sim = 1)}\cr
    Calculates the first 4 portfolio moments using the geometric properties of the model, given a
    matrix of asset weights with row dimension equal to the row dimension of the filtered 
    dataset (i.e. less any lags). The kurtosis is not returned for systems of dimensions greater 
    than 11 because of the memory issues related to calculations on the co-kurtosis tensor.
    The \dQuote{sim} option indicates the simulation index to use, given the \dQuote{m.sim} 
    option chosen in the call to the simulation function.
    }
    \item{rcokurt}{\code{signature(object = "goGARCHsim")}:\cr
    function:\cr
    \bold{rcokurt(object, from = 1, to = 1, sim = 1)}\cr
    Returns the 'time-varying'  NxN^4 cokurtosis tensor in array format and is limited to a dimensions of 
    11 assets. The \dQuote{from} and \dQuote{to} options indicate the time indices for which to return 
    the arrays. Because of memory issues, this is limited to 100 indices. The \dQuote{sim} option indicates 
    the simulation index to use, given the \dQuote{m.sim} option chosen in the call to the simulation 
    function.}
    \item{rcoskew}{\code{signature(object = "goGARCHsim")}:\cr
    function:\cr
    \bold{rcoskew(object, from = 1, to = 1, sim = 1)}\cr
    Returns the 'time-varying'  NxN^3 coskewness tensor in array format. The \dQuote{from} and \dQuote{to} 
    options indicate the time indices for which to return the arrays. Because of memory issues, 
    this is limited to 100 indices.  The \dQuote{sim} option indicates the simulation index to use, 
    given the \dQuote{m.sim} option chosen in the call to the simulation function.}
    \item{rcov}{\code{signature(object = "goGARCHsim")}: Returns the time-varying NxN covariance matrix in 
    array format.  There is an additional \dQuote{sim} option which indicates the simulation index to 
    use, given the \dQuote{m.sim} option chosen in the call to the simulation function.}
    \item{rcor}{\code{signature(object = "goGARCHforecast")}: Returns the time-varying NxN 
    correlation matrix in array format.   There is an additional \dQuote{sim} option which indicates 
    the simulation index to use, given the \dQuote{m.sim} option chosen in the call to the simulation 
    function.}
	}
}
\author{Alexios Ghalanos}
\keyword{classes}