\name{gogarchroll-methods}
\alias{gogarchroll}
\alias{gogarchroll,ANY-method}
\alias{gogarchroll,goGARCHspec-method}
\title{function: GO-GARCH Rolling Estimation}
\description{
Method for performing rolling estimation of the GO-GARCH model.
}
\usage{
gogarchroll(spec, data, n.ahead = 1, forecast.length = 50, refit.every = 25, 
refit.window = c("recursive", "moving"), parallel = FALSE, 
parallel.control = list(pkg = c("multicore", "snowfall"), cores = 2), solver = "solnp",
external.forecasts = list(mregfor = NULL), fit.control = list(), solver.control = list(), 
rseed = NULL, save.fit = FALSE, save.wdir = NULL, ...)
}
\arguments{
  \item{spec}{ A GO-GARCH spec object of class \code{\linkS4class{goGARCHspec}}.}
  \item{data}{ A multivariate data object. Can be a matrix or data.frame or timeSeries. }
  \item{n.ahead}{ The forecast horizon (only 1-ahead supported for rolling forecasts). }
  \item{forecast.length}{ The length of the total forecast for which out of sample data from 
  the dataset will be excluded for testing. }
  \item{refit.every}{ Determines every how many periods the model is re-estimated. }
  \item{refit.window}{ Whether the refit is done on an expaning window including all the previous 
  data or a moving window, the length of the window determined by the argument above 
  (\dQuote{refit.every}).}
  \item{parallel}{ Whether to make use of parallel processing on multicore systems.}
  \item{parallel.control}{ The parallel control options including the type of package for 
  performing the parallel calculations (\sQuote{multicore} for non-windows O/S and 
  \sQuote{snowfall} for all O/S), and the number of cores to make use of.}
  \item{solver}{ The solver to use. }
  \item{external.forecasts}{ A list of external.regressor forecast matrices for the 
  conditional mean equation (mregfor).}
  \item{fit.control}{ Control parameters parameters passed to the fiting function. }
  \item{solver.control}{ Control parameters passed to the solver. }
  \item{rseed}{ Initialization seed for first ICA fit. The rest of the ICA fits are initialized with
  the previous mixing matrix (using A.init). }
  \item{save.fit}{ Whether to save the fitted objects of class \code{\linkS4class{goGARCHfit}} 
  during the  estimation of each (\dQuote{refit.every}). If true, the directory to save must be 
  provided (see below). The function will not save this by default for reasons of memory management, 
  but can save it as an \dQuote{.rda} file in the user's chosen directory for further analysis.}
  \item{save.wdir}{ If \dQuote{save.fit} is true, the directory in which to save the 
  \code{\linkS4class{goGARCHfit}} objects (1 for each \dQuote{refit.every}).}
  \item{\dots}{.}
}
\value{
An object of class \code{\linkS4class{goGARCHroll}}.
}
\author{Alexios Ghalanos}
\keyword{methods}
