\name{rgarch-package}
\alias{rgarch-package}
\alias{rgarch}
\title{The rgarch package}
\description{
The rgarch package aims to provide a flexible and rich GARCH modelling and testing environment 
for the user. Modelling is a simple process of defining a specification and fitting the data. 
Inference can be made from summary, various tests and plot methods, while the forecasting, 
filtering and simulation methods complete the modelling environment. Finally, specialized 
methods are implemented for simulating the parameter distributions and the evaluating their 
consistency, and a bootstrap forecast method which takes into account both parameter and 
predictive distribution uncertainty.\cr 
The testing environment is based on a rolling backtest function which considers the more general 
context in which GARCH models are based, namely the conditional time varying estimation of density 
parameters and the implication for their use in analytical risk management measures.\cr
The mean equation allows for AR(FI)MA, arch-in-mean and external regressors, while the variance 
equation implements a wide variety of univariate garch models as well as the possibility of 
including external regressors. Finally, a set of rich distributions from the \dQuote{fBasics} 
package and Johnson's reparametrized SU from the \dQuote{gamlss} package are used for modelling 
innovations.\cr
Over time, the package will grow to add feasible multivariate GARCH models. At present, the Generalized 
Orthogonal GARCH using Independent Components Analysis (\emph{ICA}) and Dynamic Conditional 
Correlation (with multivariate Normal, Laplace and Student distributions) models are fully implemented, 
with methods for spec, fit, filter, forecast, simulation, and rolling estimation and forecasting, 
as well as specialized functions to calculate and work with the weighted portfolio conditional density.
As of version 1.85, a Copula-GARCH model is implemented with the multivariate Normal and Student 
distributions and both dynamic base (DCC) and static estimation of the correlation.\cr
}
\details{
\tabular{ll}{
Package: \tab rgarch\cr
Type: \tab Package\cr
Version: \tab 1.92\cr
Date: \tab 2011-04-20\cr
License: \tab GPL\cr
LazyLoad: \tab yes\cr
Depends: \tab R (>= 2.10.0), Rcpp (>= 0.8.5), RcppArmadillo (>= 0.2.5), methods, Matrix, numDeriv, chron, Rsolnp, sandwich, mvtnorm\cr }
While the package has implemented some safeguards, both during pre-estimation as well as the estimation phase, 
there is no guarantee of convergence in the fitting procedure. As a result, the fit method allows the user to 
input starting parameters as well as keep any parameters from the spec as fixed (including the case of all 
parameters fixed).\cr
The univariate functionality of the packages is contained in the main methods for defining a specification 
\code{\link{ugarchspec}}, fitting \code{\link{ugarchfit}}, forecasting \code{\link{ugarchforecast}}, 
simulation from fit object \code{\link{ugarchsim}}, path simulation from specification object \code{\link{ugarchpath}},
parameter distribution by simulation \code{\link{ugarchdistribution}}, bootstrap forecast \code{\link{ugarchboot}} 
and rolling estimation and forecast \code{\link{ugarchroll}}. There are also some functions which enable multiple 
fitting of assets in an easy to use wrapper with the option of multicore functionality, namely \code{\link{multispec}}, 
\code{\link{multifit}}, \code{link{multifilter}} and \code{\link{multiforecast}}. Explanations on the available methods 
for the returned classes can be found in the documentation for those classes.\cr
As of version 1.78, a separate subset of methods and classes has been included to calculate pure ARFIMA models
with constant variance. This subset includes similar functionality as with the GARCH methods, with the exception that
no plots are yet implemented, and neither is a forecast based on the bootstrap. These may be added in the future. While
there are limited examples in the documentation on the ARFIMA methods, the interested user can search the unit.test folder
of the source installation for some tests using ARFIMA models as well as equivalence to the base R arima methods (particularly
replication of simulation). Finally, no representation is made about the adequacy of ARFIMA models, particularly the 
statistical properties of parameters when using distributions which go beyond the Gaussian.\cr
The multivariate functionality, currently supports the \emph{GO-GARCH} with ICA method, and is available through 
the \code{\link{gogarchspec}}, \code{\link{gogarchfit}}, \code{\link{gogarchfilter}}, \code{\link{gogarchforecast}}, 
\code{\link{gogarchsim}} and \code{\link{gogarchroll}} functions. The \emph{DCC} with multivariate Normal,
Laplace and Student distributions is also supported with the main functionality in \code{\link{dccspec}},
\code{\link{dccfit}}, \code{\link{dccfilter}}, \code{\link{dccforecast}}, \code{\link{dccsim}} and
\code{\link{dccroll}}. The Normal and Student Copula-GARCH, with dynamic or static correlation, is implemented
with the main functionality in \code{\link{cgarchspec}}, \code{\link{cgarchfit}}, \code{\link{cgarchfilter}}, 
and \code{\link{cgarchsim}}. Usual extractor and support methods for the multivariate GARCH models are documented
in the class of the returned objects.\cr
The univariate conditional distributions used in the package are also exposed for the benefit of the user 
through the \code{\link{rgarchdist}} functions which contain methods for density, distribution, quantile, 
sampling and fitting. Additionally, the \code{\link{nigtransform}} and \code{\link{ghyptransform}} functions 
provide the necessary parameter transformation and scaling methods for moving from the location scale invariant 
\sQuote{rho-zeta} parametrization with mean and standard deviation, to the standard \sQuote{alpha-beta-delta-mu} 
parametrization of the Generalized Hyperbolic Distribution family.\cr
The type of data handled by the package is quite varied, accepting \dQuote{timeSeries}, \dQuote{xts}, 
\dQuote{zoo}, \dQuote{zooreg}, \dQuote{data.frame} with dates as rownames, \dQuote{matrix} and 
\dQuote{numeric} vector with dates as names. For the \dQuote{numeric} vector and \dQuote{data.frame} with character
dates in names or rownames, the package tries a variety of methods to try to recognize the type and format
of the date else will index the data numerically. The package holds dates internally as class \code{\link{Date}}. 
This mostly impacts the plots and forecast summary methods. For high frequency data, the user should make use of 
a non-named representation such as \dQuote{matrix} or \dQuote{numeric} as the package has yet 
to implement methods for checking and working with frequencies higher than daily. Finally, the functions
\code{\link{ForwardDates}} and \code{\link{WeekDayDummy}} offer some simple Date manipulation methods for
working with forecast dates and creating day of the week dummy variables for use in GARCH modelling.\cr
Some benchmarks (published and comparison with commercial package), are available through the 
\code{\link{ugarchbench}} function. \emph{The \sQuote{inst} folder of the source distribution also contains 
various tests which can be sourced and run by the user, also exposing some finer details of the functionality
of the package. The user should really consult the examples supplied in this folder which are quite numerous
and instructive with some comments.}
}
\section{How to cite this package}{
  Whenever using this package, please cite as\cr

 \preformatted{@Manual{Ghalanos_2011,
 author       = {Alexios Ghalanos},
 title        = {{rgarch}: Flexible GARCH modelling in R.},
 year         = {2011},
 note 	      = {R package version 1.92.},}}
\emph{}
}
\section{License}{
  The releases of this package is licensed under GPL version 3.
}
\author{Alexios Ghalanos}
\references{
Baillie, R.T. and Bollerslev, T. and Mikkelsen,H.O. \emph{Fractionally integrated generalized autoregressive conditional heteroskedasticity}, 1996, Journal of Econometrics, 3-30 .\cr
Berkowitz, J. \emph{Testing density forecasts, with applications to risk management}, 2001, Journal of Business and Economic Statistics, 19, 4, 465-474.\cr
Bollerslev, T. \emph{Generalized Autoregressive Conditional Heteroskedasticity}, 1986, Journal of Econometrics, 31, 307-327.\cr
Broda, S.A. and Paolella, M.S. \emph{CHICAGO: A Fast and Accurate Method for Portfolio Risk Calculation},
2009, Journal of Financial Econometrics 7(4), 412-436 .\cr
Croux, C. and Joossens, K. \emph{Robust estimation of the vector autoregressive model by a least trimmed squares procedure}, 2008,
COMPSTAT, 489-501.\cr
Chung, C.F. \emph{Estimating the Fractionally Integrated GARCH Model}, 1999, National Taiwan University working paper.\cr
Ding, Z., Granger, C.W.J. and Engle, R.F. \emph{A Long Memory Property of Stock Market Returns and a New Model}, 1993, Journal of Empirical Finance, 1, 83-106.\cr
de Athayde, G.M. and Flores Jr, R.G. \emph{On Certain Geometric Aspects of Portfolio Optimisation with 
Higher Moments}, 2002, mimeo.\cr
Engle, R.F. and Ng, V. K. \emph{Measuring and Testing the Impact of News on Volatility}, 1993, Journal of Finance, 48, 1749-1778.\cr
Engle, R.F. \emph{Dynamic conditional correlation}, 2002, Journal of Business and Economic Statistics, 20, 339-350.\cr
Genest, C., Ghoudi, K. and Rivest, L. \emph{A semiparametric estimation procedure of dependence
parameters in multivariate families of distributions}, 1995, Biometrika, 82, 543-552.\cr
Glosten, L.R., Jagannathan, R. and Runkle, D.E. \emph{On the Relation between the Expected Value and the Volatility of the Nominal 
Excess Return on Stocks}, 1993, Journal of Finance, vol. 48(5), 1779-1801.\cr
Hansen, B.E. \emph{Langrange Multiplier Tests for Parameter Instability in Non-Linear Models}, 1990, mimeo.\cr
Hentschel, Ludger. \emph{All in the family Nesting symmetric and asymmetric GARCH models}, 1995, Journal of Financial Economics, vol. 39, issue 1, 71-104.\cr
Joe, H. \emph{Multivariate Models and Dependence Concepts}, 1997, Chapman \& Hall, London.\cr
Jorion,P. \emph{Value at Risk}, 2001, McGraw-Hill\cr
Nelson, D.B.\emph{Conditional Heteroskedasticity in Asset Returns: A New Approach},1991, Econometrica, Vol. 59,347-370.
Pascual, L., Romo, J., Ruiz, E., \emph{Bootstrap predictive inference for ARIMA processes}, 2004,
Journal of Time Series Analysis.\cr
Pascual, L., Romo, J., Ruiz, E., \emph{Bootstrap prediction for returns and volatilities in GARCH models}, 
2006, Computational Statistics and Data Analysis.\cr
Paolella, M.S.\emph{Intermediate Probability - A Computational Approach}, 2007, Wiley-Interscience.\cr
Schmidt, R., Hrycej, T. and Stutzle \emph{Multivariate distribution models with generalized hyperbolic 
margins}, 2006, Computational Statistics \& Data Analysis 50(8), 2065-2096.\cr
Vlaar, P.J.G. and Palm, F.C. \emph{The Message in Weekly Exchange Rates in the European Monetary System: Mean Reversion, 
Conditional Heteroskedasticity and Jumps}, 1993, Journal of Business and Economic Statistics, 11, 351-360.\cr
}