\name{varxfilter}
\alias{varxfilter}
\title{
VARX Fit/Filter Function
}
\description{
Vector Autoregressive (VAR) with Constant and Optional Exogenous Regressors (X)
Fit and Filter function for use with multivariate GARCH models.
}
\usage{
varxfilter(X, p, exogen = NULL, Bcoef = NULL, robust = FALSE, gamma = 0.25, delta = 0.01, nc = 10, ns = 500)
}
\arguments{
  \item{X}{
  A multivariate data matrix.
}
  \item{p}{
  The number of autoregressive lags.
}
  \item{exogen}{
  An optional matrix of exogenous regressors with as many rows as X, and appropriately lagged.
}
  \item{Bcoef}{
  As optional matrix of coefficients for the VARX model to filter rather than fit X.
}
  \item{robust}{
  Whether to use the robust version of VAR based on the multivariate Least Trimmed Squares Estimator 
	described in Croux and Joossens (2008).
}
 \item{gamma}{
 Proportion to trim in the robust method.
 }
  \item{delta}{
 The critical value for Reweighted estimator for the robust method.
 }
  \item{ns}{
 The number of subsets to use for the robust method.
 }
  \item{nc}{
 The number of C-steps to use for the robust method.
 }
}
\details{
This is a convenience function to be optionally used when using the multivariate GARCH methods.
}
\value{
A list with the following items:\cr
\item{Bcoef }{The coefficient matrix with rows equal to number of assets, and columns equal to
number of assets x number of lags plus 1 (constant) plus number of exogenous regressors.}
\item{xfitted }{The fitted series (conditional mean series).}
\item{xresiduals }{The residuals.}
\item{Bcov }{The covariance matrix of the coefficients.}
\item{se }{The standard error of the coefficients.} 
\item{tstat }{The t-stat of the s.e.}
\item{pstat }{The p-values of the s.e.}
\item{lag }{The number of autoregressive lags.}
\item{mxn }{The number of exogenous regressors.}
}
\references{
Lutkepohl, H. \emph{New introduction to multiple time series analysis}, 2005, Springer.\cr
Croux, C. and Joossens, K. \emph{Robust estimation of the vector autoregressive model by a least trimmed squares procedure}, 2008,
COMPSTAT, 489-501.\cr
}
\author{Alexios Ghalanos}
\note{
Part of the functionality and structure is borrowed from the 'vars' package, but the estimation 
method is implemented in a different way without calling 'lm'. Internally, methods for VARX 
forecasting and simulation are also implemented but not currently exposed to the user directly. 
The robust method is based on the matlab program of Christophe Croux available from his website.
}