/*################################################################################
##
##   R package rgarch by Alexios Ghalanos Copyright (C) 2009, 2010, 2011
##   This file is part of the R package rgarch.
##
##   The R package rgarch is free software: you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   The R package rgarch is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
#################################################################################*/
# include <R.h>
# include <math.h>
# include "rgarch.h"
# include "filters.h"
# include "distributions.h"

/* GARCH (p,q) Filter with ARMA, ARFIMA, Exogenous Regressors, and GARCH-IN-MEAN
 * Option for Mean Equation norm, skew-norm, t, skew-t, ged, skew-ged, nig & jsu
 * distributions for innovations*/
void arfimafitC(int *armap, int *armaq, int *arfima, int *incmex, double *mu, double *ar, double *ma,
		double *mxreg, double *sigma, double *dlambda, double *skew, double *shape, 
		int *dist, double *x, double *res, double *mexdata, double *zrf, double *constm, 
		double *condm, int *m, int *T, double*h, double *z, double *llh, double *LHT)
{
	int i;
	double lk=0;
	for(i=0; i<*m; i++)
	{
		arfimaxfilter(*armap, *armaq, *arfima, 0, *incmex, *mu, ar, ma, 0, mxreg, x, res, mexdata, zrf, constm,
				condm, h, *m, i, *T);
		z[i] = res[i]/fabs(*sigma);
		LHT[i] = log(garchdistribution(z[i], sqrt(fabs(*sigma)), *skew, *shape, *dlambda, *dist));
		lk = lk - LHT[i];
	}
	for (i=*m; i<*T; i++)
	{
		arfimaxfilter(*armap, *armaq, *arfima, 0, *incmex, *mu, ar, ma, 0, mxreg, x, res, mexdata, zrf, constm, condm, h, *m, i, *T);
		z[i] = res[i]/fabs(*sigma);
		LHT[i] = log(garchdistribution(z[i], fabs(*sigma), *skew, *shape, *dlambda, *dist));
		lk = lk - LHT[i];
	}
	*llh=lk;
}
