/*################################################################################
##
##   R package rgarch by Alexios Ghalanos Copyright (C) 2009, 2010, 2011
##   This file is part of the R package rgarch.
##
##   The R package rgarch is free software: you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   The R package rgarch is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
#################################################################################*/

#include "copulas.h"
using namespace Rcpp;


SEXP copulaNormalC1(SEXP Rbar, SEXP Z, SEXP dnormZ)
{
	try {
		Rcpp::NumericMatrix xRbar(Rbar);
		Rcpp::NumericMatrix xZ(Z);
		Rcpp::NumericMatrix xdnormZ(dnormZ);
		int nc = xRbar.nrow(), nr = xZ.nrow(), j;
		List output(2);
		NumericVector llhout(nr);
		arma::mat Rb(xRbar.begin(), nc, nc, true);
		arma::mat Zb(xZ.begin(), nr, nc, true);
		arma::mat dnormZb(xdnormZ.begin(), nr, nc, true);
		double llhtemp = 0.0, llh = 0.0, temp1 = 0.0, temp2 = 0.0, temp4 = 0.0;
		double lcons = nc*log(2.0*PI);
		arma::mat tempR1(nc, nc);
		tempR1 = arma::inv(Rb);
		temp1 = log(arma::det(Rb));
		for(j=0; j<nr; j++)
		{
			temp2 = 0.0;
			temp4 = 0.0;
			llhtemp = 0.0;
			temp2 = arma::as_scalar( Zb.row(j) * ( tempR1 * arma::trans(Zb.row(j))) );
			temp4 = arma::sum( dnormZb.row(j) );
			llhtemp =  lcons + temp1 + temp2 - temp4;
			llh+= llhtemp;
			llhout[j] = -0.5 * llhtemp;
		}
		output[0] = llhout;
		output[1] = -0.5 * llh;
	return(output);
	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
	} catch(...) {
		::Rf_error( "rgarch-->cgarchfit c++ exception (unknown reason)" );
	}
		return R_NilValue;
}

SEXP copulaNormalC2(SEXP Qbar, SEXP Z, SEXP dnormZ, SEXP dcca, SEXP dccb, SEXP dccorder, SEXP dccsum)
{
	try {
		Rcpp::NumericMatrix xQbar(Qbar);
		Rcpp::NumericMatrix xZ(Z);
		Rcpp::NumericMatrix xdnormZ(dnormZ);
		int nc = xQbar.nrow(), nr = xZ.nrow(), i, j;
		List output(4), Qtout(nr), Rtout(nr);
		NumericVector llhout(nr);
		arma::mat Qb(xQbar.begin(), nc, nc, true);
		arma::mat Zb(xZ.begin(), nr, nc, true);
		arma::mat dnormZb(xdnormZ.begin(), nr, nc, true);
		double *a = REAL(dcca);
		double *b = REAL(dccb);
		// ddcP = dcc arch, dccQ = dcc GARCH
		double *timp = REAL(dccorder);
		int dccP = (int) timp[0];
		int dccQ = (int) timp[1];
		int m = (int) timp[2];
		//int dccP=1, dccQ=1, m=1;
		// sum(a+b)
		double *sba = REAL(dccsum);
		double llhtemp = 0.0, llh = 0.0, temp2 = 0.0, temp4 = 0.0;
		double lcons = nc*log(2.0*PI);
		// empty matrices
		arma::mat Qt(nc, nc);
		arma::mat Rt(nc, nc);
		arma::mat UQ(nc, nc);
		arma::mat temp1(nc, nc);
		// unconditional correlation
		// Unconditiona/intercept
		UQ = Qb * (1.0 - sba[0]);
		// rem: newmat matrices start at one not zero
		// populate Dt/Dtlist
		for(j=0; j<m; j++)
		{
			Qtout[j] = Qb;
			llhout[j] = 0.0;
		}
		for(j=m; j<nr; j++)
		{
			temp1.zeros();
			temp2 = 0.0;
			temp4 = 0.0;
			llhtemp = 0.0;
			Rt.zeros();
			Qt = UQ;
			for(i = 0; i<dccP; i++)
			{
				Qt = Qt + a[i]*( arma::trans( Zb.row(j-(i+1)) ) * Zb.row(j-(i+1)) );
			}
			for(i = 0; i<dccQ; i++)
			{
				arma::mat Qtemp = Rcpp::as<arma::mat>(Qtout[j-(i+1)]);
				Qt = Qt + b[i] *Qtemp;
			}
			Qtout[j] = Qt;
			temp1 = arma::sqrt(Qt.diag()) * arma::trans(arma::sqrt(Qt.diag()));
			Rt = Qt / temp1;
			Rtout[j] = Rt;
			temp2 = arma::as_scalar( Zb.row(j) * (arma::inv(Rt) * arma::trans(Zb.row(j))));
			temp4 = arma::sum( dnormZb.row(j) );
			llhtemp = lcons + log(arma::det(Rt)) + temp2 - temp4;
			llh+= llhtemp;
			llhout[j] = -0.5 * llhtemp;
		}
		output[0] = Qtout;
		output[1] = llhout;
		output[2] = -0.5 * llh;
		output[3] = Rtout;
		return(output);
	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    	} catch(...) {
		::Rf_error( "rgarch-->cgarchfit c++ exception (unknown reason)" );
    	}
    	return R_NilValue;
}

SEXP copulaStudentC1(SEXP Rbar, SEXP Z, SEXP dtZ, SEXP nu)
{
	try {
		// dtZ is dt(Z, nu, log = TRUE)
		Rcpp::NumericMatrix xRbar(Rbar);
		Rcpp::NumericMatrix xZ(Z);
		Rcpp::NumericMatrix xdtZ(dtZ);
		int nc = xRbar.nrow(), nr = xZ.nrow(), j;
		List output(2);
		NumericVector llhout(nr);
		arma::mat Rb(xRbar.begin(), nc, nc, true);
		arma::mat Zb(xZ.begin(), nr, nc, true);
		arma::mat dtZb(xdtZ.begin(), nr, nc, true);
		double *v = REAL(nu);
		double llhtemp = 0.0, llh = 0.0, temp1 = 0.0, temp2 = 0.0, temp4 = 0.0;
		double lcons = Rf_lgammafn(0.5 * (v[0] + nc)) - Rf_lgammafn(0.5 * v[0]) - 0.5 * nc * log(PI*v[0]);
		arma::mat tempR1(nc, nc);
		tempR1 = arma::inv(Rb);
		temp1 = log(arma::det(Rb));
		for(j=0; j<nr; j++)
		{
			temp2 = 0.0;
			temp4 = 0.0;
			llhtemp = 0.0;
			temp2 = arma::as_scalar( Zb.row(j) * (tempR1 * arma::trans(Zb.row(j))));
			temp4 = arma::sum( dtZb.row(j) );
			llhtemp = lcons - 0.5 * temp1 - 0.5 * (v[0] + nc) * log(1.0 + (1.0/(v[0])) * temp2) - temp4;
			llh+= llhtemp;
			llhout[j] = llhtemp;
		}
		output[0] = llhout;
		output[1] = llh;
		return(output);
	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    	} catch(...) {
		::Rf_error( "rgarch-->cgarchfit c++ exception (unknown reason)" );
    	}
    	return R_NilValue;
}


SEXP copulaStudentC2(SEXP Qbar, SEXP Z, SEXP dtZ, SEXP dcca, SEXP dccb, SEXP nu, SEXP dccorder, SEXP dccsum)
{
	try {
		Rcpp::NumericMatrix xQbar(Qbar);
		Rcpp::NumericMatrix xZ(Z);
		Rcpp::NumericMatrix xdtZ(dtZ);
		int nc = xQbar.nrow(), nr = xZ.nrow(), i, j;
		List output(4), Qtout(nr), Rtout(nr);
		NumericVector llhout(nr);
		arma::mat Qb(xQbar.begin(), nc, nc, true);
		arma::mat Zb(xZ.begin(), nr, nc, true);
		arma::mat dtZb(xdtZ.begin(), nr, nc, true);
		// nr is already (nr(data) + m)
		double *a = REAL(dcca);
		double *b = REAL(dccb);
		double *v = REAL(nu);
		// ddcP = dcc arch, dccQ = dcc GARCH
		double *timp = REAL(dccorder);
		int dccP = (int) timp[0];
		int dccQ = (int) timp[1];
		int m = (int) timp[2];
		//int dccP=1, dccQ=1, m=1;
		// sum(a+b)
		double *sba = REAL(dccsum);
		double llhtemp = 0.0, llh = 0.0, temp2 = 0.0, temp4 = 0.0;
		double lcons = Rf_lgammafn(0.5 * (v[0] + nc)) - Rf_lgammafn(0.5 * v[0]) - 0.5 * nc * log(PI*v[0]);
		// empty matrices
		arma::mat Qt(nc, nc);
		arma::mat Rt(nc, nc);
		arma::mat UQ(nc, nc);
		arma::mat temp1(nc, nc);
		// unconditional correlation
		// Unconditiona/intercept
		UQ = Qb * (1.0 - sba[0]);
		// rem: newmat matrices start at one not zero
		// populate Dt/Dtlist
		for(j=0; j<m; j++)
		{
			Qtout[j] = Qb;
			llhout[j] = 0.0;
		}
		for(j=m; j<nr; j++)
		{
			temp1.zeros();
			temp2 = 0.0;
			temp4 = 0.0;
			llhtemp = 0.0;
			Rt.zeros();
			Qt = UQ;
			for(i = 0; i<dccP; i++)
			{
				Qt = Qt + a[i]*( arma::trans( Zb.row(j-(i+1)) ) * Zb.row(j-(i+1)) );
			}
			for(i = 0; i<dccQ; i++)
			{
				arma::mat Qtemp = Rcpp::as<arma::mat>(Qtout[j-(i+1)]);
				Qt = Qt + b[i] *Qtemp;
			}
			Qtout[j] = Qt;
			temp1 = arma::sqrt(Qt.diag()) * arma::trans(arma::sqrt(Qt.diag()));
			Rt = Qt / temp1;
			Rtout[j] = Rt;
			temp2 = arma::as_scalar( Zb.row(j) * (arma::inv(Rt) * arma::trans(Zb.row(j))));
			temp4 = arma::sum( dtZb.row(j) );
			llhtemp = lcons - 0.5 * log(arma::det(Rt)) - 0.5 * (v[0] + nc) * log(1.0 + (1.0/v[0]) * temp2) - temp4;
			llh+= llhtemp;
			llhout[j] = llhtemp;
		}
		output[0] = Qtout;
		output[1] = llhout;
		output[2] = llh;
		output[3] = Rtout;
		return(output);

	} catch( std::exception &ex ) {
		forward_exception_to_r( ex );
    	} catch(...) {
		::Rf_error( "rgarch-->cgarchfit c++ exception (unknown reason)" );
    	}
    	return R_NilValue;
}

