/*################################################################################
##
##   R package rgarch by Alexios Ghalanos Copyright (C) 2009, 2010, 2011
##   This file is part of the R package rgarch.
##
##   The R package rgarch is free software: you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   The R package rgarch is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
#################################################################################*/
# include <R.h>
# include <math.h>
# include "filters.h"

void sgarchfilter(double omega, double *alpha, double *beta, double *vexdata,
		double *vxreg, int incvex, int T, double *e, int i, int garchp,
		int garchq, double *h)
{
	int j, ind;
	h[i] = h[i] + omega;
	if( incvex>0 )
	{
		ind=0;
		for( j=0; j<incvex; j++ )
		{
			ind = i + ( T * j );
			h[i] = h[i] + vxreg[j]*vexdata[i];
		}
	}

	for( j=0; j<garchp; j++ )
	{
		h[i] = h[i] + alpha[j]*e[i-(j+1)];
	}
	for( j=0; j<garchq; j++ )
	{
		h[i] = h[i] + beta[j]*h[i-(j+1)];
	}
}

void gjrgarchfilter(double omega, double *alpha, double *gm, double *beta,
		double *vexdata, double *vxreg, int incvex, int T, double *e,
		double *nres, int i, int garchp, int garchq, double *h)
{
	int j, ind;
	h[i] = h[i] + omega;
	if( incvex>0 )
	{
		ind = 0;
		for( j=0; j<incvex; j++ )
		{
			ind = i + ( T*j );
			h[i]+= vxreg[j]*vexdata[ind];
		}
	}
	for( j=0; j<garchp; j++ )
	{
		h[i]+= alpha[j]*( e[i-(j+1)] ) + gm[j]*nres[i-(j+1)];
	}
	for( j=0; j<garchq; j++ )
	{
		h[i]+= beta[j]*h[i-(j+1)];
	}
}

void aparchfilter(double omega, double *alpha, double *gm, double *beta,
		double delta, double *vexdata, double *vxreg, int incvex, int T,
		double *res, int i, int garchp, int garchq, double *h)
{
	int j, ind;
	h[i] = h[i] + omega;
	if( incvex>0 )
	{
		ind=0;
		for( j=0; j<incvex; j++ )
		{
			ind = i + ( T*j );
			h[i]+= vxreg[j] * vexdata[ind];
		}
	}
	for( j=0; j<garchp; j++ )
	{
		h[i]+= alpha[j]* pow(fabs(res[i-(j+1)])-gm[j]*res[i-(j+1)], delta);
	}
	for( j=0; j<garchq; j++ )
	{
		h[i]+= beta[j]*pow( h[i-(j+1)], delta );
	}
	h[i] = pow( h[i], 1.0/delta );
}

void egarchfilter(double omega, double *alpha, double meanz, double *gm,
		double *beta, double *z, double *vexdata, double *vxreg, int incvex, int T,
		int i, int garchp, int garchq, double *h)
{
	int j, ind;
	h[i] = h[i] + omega;
	if( incvex>0 )
	{
		ind=0;
		for( j=0; j<incvex; j++ )
		{
			ind = i + ( T*j );
			h[i] = h[i] + vxreg[j]*vexdata[ind];
		}
	}
	for( j=0; j<garchp; j++ )
	{

		h[i] = h[i] + alpha[j]*z[i-(j+1)] + gm[j]*( fabs(z[i-(j+1)]) - meanz );
	}
	for( j=0; j<garchq; j++ )
	{
		h[i] = h[i] + beta[j]*log( h[i-(j+1)] );
	}
	h[i] = exp( h[i] );
}

void fgarchfilter(double omega, double *alpha, double *gamma1, double *gamma2, double kdelta,
		double *beta, double lambda, double *z, double *vexdata, double *vxreg,
		int incvex, int T, int i, int garchp, int garchq, double *h)
{
	int j, ind;
	h[i] = h[i] + omega;
	if( incvex>0 )
	{
		ind=0;
		for( j=0; j<incvex; j++ )
		{
			ind = i + ( T*j );
			h[i]= h[i] + vxreg[j] * vexdata[ind];
		}
	}
	for( j=0; j<garchp; j++ )
	{
		h[i]= h[i] + alpha[j] * ( pow(sqrt(pow(0.001,2) + pow( z[i-(j+1)] - gamma2[j],2) )- gamma1[j] *(z[i-(j+1)] - gamma2[j]), kdelta ) * pow( h[i-(j+1)], lambda) );
	}
	for( j=0; j<garchq; j++ )
	{
		h[i]= h[i] + beta[j] * pow( h[i-(j+1)], lambda );
	}
	h[i] = pow( h[i], 1/lambda );
}


void arfimaxfilter(int armap, int armaq, int arfima, double im, int incmex,
		double mu, double *ar, double *ma, double inmean, double *mxreg, double *x,
		double *res, double *mexdata, double *zrf, double *constm, double *condm,
		double *h, int m, int i, int T)
{
/* --------------------------------------------------------------------------------
 * ARFIMA Process :
 * (1-L)^(-darfima).e[t] = phi(1-L)(y[t] - mu[t]) - psi(L).e[t]
 * where mu[t] includes constant, external and arch-in-mean
 * L is the lag operator
 * --------------------------------------------------------------------------------
 * */
	/*0 constm, 1 condm, 2 res*/
	int j, k, ind;
	constm[i] = mu;
	// GARCH-In-Mean Initialization
	if(im>0)
	{
		constm[i]+=inmean*pow(h[i], im);
	}
	// Exogenous Regressor Initialization
	if(incmex>0)
	{
		ind=0;
		for(k=0;k<incmex;k++)
		{
			ind=i+(T*k);
			constm[i]+=mxreg[k]*mexdata[ind];
		}
	}
	condm[i]+=constm[i];
	//ARMA initialization
	if(armap!=0 || armaq!=0)
	{
		if(i<armap)
		{
				condm[i]=x[i];
		}
		else
			{
				if(armap!=0)
				{
						for(j=0; j<armap;j++)
						{
							condm[i]+=ar[j]*(x[i-(j+1)]-constm[i-(j+1)]);
						}
				}
				if(armaq!=0)
				{
					for(j=0; j<armaq;j++)
					{
						if(i-j-1>=0)
						{
							condm[i]+=ma[j]*(x[i-(j+1)]-condm[i-(j+1)]);
						}
					}
				}
			}
	}
	res[i]=x[i]-condm[i];
	//arfima initialization
	if(arfima>0)
	{
		if(i>0 && i<m)
		{
			double tmp=0;
			for(k=1;k<=i;k++)
			{
				tmp+=(zrf[i-k+1]*res[k-1]) ;
			}
			res[i]=-1.0 * tmp;
		}
		if(i>0 && i>=m)
		{
			double tmp=0;
			for(k=i;k>0;k--)
			{
				// quicker to count down
				tmp+=zrf[k]*(x[i-k] - condm[i-k]) ;
			}
			res[i]+= tmp;
		}
	}
}

/* We are passing (1-sum(ar))*mu in constm (this is not the case in fracsim).
 * r[t] = (1-ar)*mu[t] + ar*(r[t-1]) + ma*(res[t-1]) + res[t]
 * r[t] - mu[t]  = ar*(r[t-1]-mu[t]) + ma*(res[t-1]) + res[t]
 * */
void armaxsim(int *armap, int *armaq, double *constm, double *ar, double *ma, double *x,
		double *res, int *m, int *T)
{
	int j, i;
	for(i=*m; i<*T; i++)
	{
		x[i] = constm[i];
		for( j=0; j<*armap; j++ )
		{
			x[i]+= ar[j] * (x[i-(j+1)] - constm[i-(j+1)]);
		}
		for ( j=0; j<*armaq; j++ )
		{
			x[i]+= ma[j] * res[i-(j+1)];
		}
		x[i]+= res[i];
	}
}

