/*################################################################################
##
##   R package rgarch by Alexios Ghalanos Copyright (C) 2009, 2010, 2011
##   This file is part of the R package rgarch.
##
##   The R package rgarch is free software: you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   The R package rgarch is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
#################################################################################*/
# include <R.h>
# include <math.h>
# include "rgarch.h"
# include "filters.h"
# include "distributions.h"

/* GJR-GARCH (p,q) Filter with ARMA, ARFIMA, Exogenous Regressors, and GARCH-IN-MEAN
 * Option for Mean Equation norm, skew-norm, t, skew-t, ged, skew-ged, nig & jsu
 * distributions for innovations*/

void gjrgarchfilterC(int *armap, int *armaq, int *arfima, double *im, int *incmex,
		int *garchp, int *garchq, int *incvex, double *mu, double *ar, double *ma,
		double *inmean, double *mxreg, double *omega, double *hEst, double *alpha,
		double *beta, double *gm, double *vxreg, double *dlambda, double *skew, double *shape,
		int *dist, double *x, double *res, double *nres, double *e, double *mexdata,
		double *vexdata, double *zrf, double *constm, double *condm, int *m, int *T,
		double *h, double *z, double *llh, double *LHT)
{
	int i;
	double lk=0;
	for( i=0; i<*m; i++ )
	{
		h[i] = *hEst;
		arfimaxfilter(*armap, *armaq, *arfima, *im, *incmex, *mu,
				ar, ma, *inmean, mxreg, x, res, mexdata, zrf, constm,
				condm, h, *m, i, *T);
		e[i] = res[i] * res[i];
		nres[i] = res[i] < 0.0 ? e[i] : 0.0;
		LHT[i] = log(garchdistribution(z[i], sqrt(fabs(h[i])), *skew, *shape, *dlambda, *dist));
		lk = lk - LHT[i];
	}
	// Main Loop
	for ( i=*m; i<*T; i++ )
	{
		gjrgarchfilter(*omega, alpha, gm, beta, vexdata, vxreg, *incvex, *T, e,
				nres, i, *garchp, *garchq, h);
		arfimaxfilter(*armap, *armaq, *arfima, *im, *incmex, *mu,
				ar, ma, *inmean, mxreg, x, res, mexdata, zrf, constm,
				condm, h, *m, i, *T);
		e[i] = res[i] * res[i];
		nres[i] = res[i] < 0.0 ? e[i] : 0.0;
		z[i] = res[i]/sqrt(fabs(h[i]));
		LHT[i] = log(garchdistribution(z[i], sqrt(fabs(h[i])), *skew, *shape, *dlambda, *dist));
		lk = lk - LHT[i];
	}
	*llh = lk;
}

void gjrgarchsimC(int *garchp, int *garchq, int *incvex, double *omega, double *alpha, double *beta,
		double *gm, double *vxreg, double *h, double *z, double *res, double *e, double *nres,
		double *vexdata, int *T, int *m)
{
	int i;
	// first the variance equation
	for ( i=*m; i<*T; i++ )
	{
		gjrgarchfilter(*omega, alpha, gm, beta, vexdata, vxreg, *incvex, *T, e,
				nres, i, *garchp, *garchq, h);
		res[i] = pow( h[i],0.5 ) * z[i];
		e[i] = res[i] * res[i];
		nres[i] = res[i] < 0.0 ? e[i] : 0.0;
	}
}
